
alter session set current_schema = TFG_GPOSESMZ;

/**
************************* Tabla Campaña *************************
*/
CREATE TABLE CAMPANHA (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(250) NOT NULL, 
 FECHA_INICIO DATE NOT NULL,
 FECHA_FIN DATE NOT NULL, 
 IDLIDER NUMBER NOT NULL,  
 CONSTRAINT PK_CAMPANHA_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX CAMPANHA_PK ON CAMPANHA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE CAMPANHA
ADD CONSTRAINT UQ_CAMPANHA_NOMBRE_FECHAS UNIQUE (NOMBRE, FECHA_INICIO, FECHA_FIN)
ENABLE;
COMMENT ON COLUMN CAMPANHA.ID IS 'Identificador de la campaña.';
COMMENT ON COLUMN CAMPANHA.NOMBRE IS 'Nombre de la campaña.';
COMMENT ON COLUMN CAMPANHA.FECHA_INICIO IS 'Fecha de inicio de la campaña.';
COMMENT ON COLUMN CAMPANHA.FECHA_FIN IS 'Fecha de fin de la campaña.';
COMMENT ON COLUMN CAMPANHA.IDLIDER IS 'Identificador de la persona responsable.';

/**
************************* Tabla Auditoría *************************
*/
CREATE TABLE AUDITORIA (
 ID NUMBER NOT NULL, 
 DESCRIPCION VARCHAR2(250) NOT NULL,
 COSTE DECIMAL(10,2) DEFAULT 0 NOT NULL,
 IDCAMPANHA NUMBER NOT NULL, 
 IDEMPRESA NUMBER NOT NULL, 
 IDEMPRESAEXTERNA NUMBER, 
 FECHA_INICIO DATE NOT NULL,
 CONSTRAINT PK_AUDITORIA_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX AUDITORIA_PK ON AUDITORIA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN AUDITORIA.ID IS 'Identificador de la auditoría.';
COMMENT ON COLUMN AUDITORIA.DESCRIPCION IS 'Descripción de la auditoría.';
COMMENT ON COLUMN AUDITORIA.COSTE IS 'Coste de la auditoría.';
COMMENT ON COLUMN AUDITORIA.FECHA_INICIO IS 'Fecha de inicio de la auditoría.';
COMMENT ON COLUMN AUDITORIA.IDEMPRESA IS 'Identificador de la empresa a la que pertenece la auditoría.';
COMMENT ON COLUMN AUDITORIA.IDEMPRESAEXTERNA IS 'Identificador de la empresa externa que realiza la auditoría.';

/**
************************* Tabla Empresa *************************
*/
CREATE TABLE EMPRESA (
 ID NUMBER NOT NULL, 
 CIF VARCHAR2(50) NOT NULL, 
 NOMBRE VARCHAR2(100) NOT NULL, 
 CONSTRAINT PK_EMPRESA_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX EMPRESA_PK ON EMPRESA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
ALTER TABLE EMPRESA
ADD CONSTRAINT UQ_EMPRESA_CIF UNIQUE (CIF)
ENABLE;
COMMENT ON COLUMN EMPRESA.ID IS 'Identificador de la empresa.';
COMMENT ON COLUMN EMPRESA.CIF IS 'Identificador fiscal de la empresa.';
COMMENT ON COLUMN EMPRESA.NOMBRE IS 'Nombre de la empresa.';

/**
************************* Tabla Empresa externa *************************
*/
CREATE TABLE EMPRESA_EXTERNA (
 ID NUMBER NOT NULL, 
 CIF VARCHAR2(50) NOT NULL, 
 NOMBRE VARCHAR2(100) NOT NULL, 
 CONSTRAINT PK_EMPRESA_EXTERNA_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX EMPRESA_EXTERNA_PK ON EMPRESA_EXTERNA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
ALTER TABLE EMPRESA_EXTERNA
ADD CONSTRAINT EMPRESA_EXTERNA_UK1 UNIQUE (CIF)
ENABLE;
COMMENT ON COLUMN EMPRESA_EXTERNA.ID IS 'Identificador de la empresa externa.';
COMMENT ON COLUMN EMPRESA_EXTERNA.CIF IS 'Identificador fiscal de la empresa externa.';
COMMENT ON COLUMN EMPRESA_EXTERNA.NOMBRE IS 'Nombre de la empresa externa.';

/**
************************* Tabla Departamento *************************
*/
CREATE TABLE DEPARTAMENTO (
 ID NUMBER NOT NULL,
 NOMBRE VARCHAR2(50) NOT NULL, 
 IDEMPRESA NUMBER NOT NULL, 
 CONSTRAINT PK_DEPARTAMENTO_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX DEPARTAMENTO_PK ON DEPARTAMENTO (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
ALTER TABLE DEPARTAMENTO
ADD CONSTRAINT UQ_DEPARTAMENTO_NOMBRE_IDEMPRESA UNIQUE (NOMBRE, IDEMPRESA)
ENABLE;
COMMENT ON COLUMN DEPARTAMENTO.ID IS 'Identificador del departamento.';
COMMENT ON COLUMN DEPARTAMENTO.NOMBRE IS 'Nombre del departamento.';
COMMENT ON COLUMN DEPARTAMENTO.IDEMPRESA IS 'Identificador de la empresa al que pertenece.';

/**
************************* Tabla Auditoria Departamento *************************
*/
CREATE TABLE AUDITORIA_DEPARTAMENTO (
 IDAUDITORIA NUMBER NOT NULL, 
 IDDEPARTAMENTO NUMBER NOT NULL 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN AUDITORIA_DEPARTAMENTO.IDAUDITORIA IS 'Identificador de la auditoría.';
COMMENT ON COLUMN AUDITORIA_DEPARTAMENTO.IDDEPARTAMENTO IS 'Identificador del departamento.';

/**
************************* Tabla Control *************************
*/
CREATE TABLE CONTROL (
 ID NUMBER NOT NULL, 
 DESCRIPCION VARCHAR2(250) NOT NULL, 
 IDAUDITORIA NUMBER NOT NULL, 
 IDCREADOR NUMBER NOT NULL, 
 IDEJECUTOR NUMBER NOT NULL, 
 IDRIESGO NUMBER, 
 CONSTRAINT PK_CONTROL_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX CONTROL_PK ON CONTROL (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN CONTROL.ID IS 'Identificador del control.';
COMMENT ON COLUMN CONTROL.DESCRIPCION IS 'Descripción del control a ejecutar.';
COMMENT ON COLUMN CONTROL.IDAUDITORIA IS 'Identificador de la auditoría a la que pertenece el control.';
COMMENT ON COLUMN CONTROL.IDCREADOR IS 'Identificador de la persona que crea el control.';
COMMENT ON COLUMN CONTROL.IDEJECUTOR IS 'Identificador de la persona que ejecuta el control.';
COMMENT ON COLUMN CONTROL.IDRIESGO IS 'Identificador del riesgo asociado al control.';

/**
************************* Tabla Muestreo *************************
*/
CREATE TABLE MUESTREO (
 ID NUMBER NOT NULL, 
 DESCRIPCION VARCHAR2(100) NOT NULL, 
 RESULTADO VARCHAR2(50) DEFAULT 'NO_EJECUTADO' NOT NULL, 
 OBSERVACIONES VARCHAR2(1000), 
 IDCONTROL NUMBER NOT NULL, 
 IDRESPONSABLE NUMBER NOT NULL, 
 CONSTRAINT PK_MUESTREO_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX MUESTREO_PK ON MUESTREO (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
ALTER TABLE MUESTREO
ADD CONSTRAINT CHK_MUESTREO_RESULTADO CHECK (RESULTADO IN ('NO_EJECUTADO', 'EN_EJECUCION', 'OK', 'KO'))
ENABLE;
COMMENT ON COLUMN MUESTREO.ID IS 'Identificador de la muestra.';
COMMENT ON COLUMN MUESTREO.DESCRIPCION IS 'Descripción de la muestra a realizar.';
COMMENT ON COLUMN MUESTREO.RESULTADO IS 'Resultado de la muestra. Solo se aceptan valores del enumerado Resultado (NO_EJECUTADO, EN_EJECUCION, OK, KO).';
COMMENT ON COLUMN MUESTREO.OBSERVACIONES IS 'Texto de observaciones.';
COMMENT ON COLUMN MUESTREO.IDCONTROL IS 'Identificador del control relacionado a la muestra.';
COMMENT ON COLUMN MUESTREO.IDRESPONSABLE IS 'Identificador de la persona responsable a la muestra.';

/**
************************* Tabla Persona *************************
*/
CREATE TABLE PERSONA (
 ID NUMBER NOT NULL, 
 NIF VARCHAR2(30) NOT NULL, 
 NOMBRE VARCHAR2(200) NOT NULL, 
 IDEMPRESA NUMBER, 
 IDEMPRESAEXTERNA NUMBER, 
 IDDEPARTAMENTO NUMBER NOT NULL, 
 IDORIGEN NUMBER NOT NULL, 
 CONSTRAINT PK_PERSONA_ID PRIMARY KEY (ID)
 USING INDEX(CREATE UNIQUE INDEX PERSONA_PK ON PERSONA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN PERSONA.ID IS 'Identificador de la persona.';
COMMENT ON COLUMN PERSONA.NIF IS 'Identificador fiscal de la persona.';
COMMENT ON COLUMN PERSONA.NOMBRE IS 'Nombre de la persona.';
COMMENT ON COLUMN PERSONA.IDEMPRESA IS 'Identificador de la empresa donde trabaja la persona ( relación XOR con EmpresaExterna).';
COMMENT ON COLUMN PERSONA.IDEMPRESAEXTERNA IS 'Identificador de la empresa externa donde trabaja la persona ( relación XOR con Empresa).';
COMMENT ON COLUMN PERSONA.IDDEPARTAMENTO IS 'Identificador del departamento donde trabaja la persona.';
COMMENT ON COLUMN PERSONA.IDORIGEN IS 'Identificador del equipo de origen de la persona.';

/**
************************* Tabla Accion *************************
*/
CREATE TABLE ACCION (
 ID NUMBER NOT NULL, 
 DESCRIPCION VARCHAR2(200) NOT NULL, 
 FECHA_CREACION DATE NOT NULL, 
 FECHA_ESTIMADA_EJECUCION DATE, 
 FECHA_REAL_EJECUCION DATE, 
 IDCREADOR NUMBER NOT NULL, 
 IDRESPONSABLE NUMBER NOT NULL, 
 IDESTADO NUMBER NOT NULL, 
 IDRIESGO NUMBER NOT NULL, 
 CONSTRAINT PK_ACCION_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX ACCION_PK ON ACCION (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN ACCION.ID IS 'Identificador de la acción.';
COMMENT ON COLUMN ACCION.DESCRIPCION IS 'Descripción de la acción.';
COMMENT ON COLUMN ACCION.FECHA_CREACION IS 'Fecha de creación.';
COMMENT ON COLUMN ACCION.FECHA_ESTIMADA_EJECUCION IS 'Fecha estimada de la ejecución.';
COMMENT ON COLUMN ACCION.FECHA_REAL_EJECUCION IS 'Fecha real de ejecución.';
COMMENT ON COLUMN ACCION.IDCREADOR IS 'Identificador de la persona creadora de la acción.';
COMMENT ON COLUMN ACCION.IDRESPONSABLE IS 'Identificador de la persona responsable de la acción.';
COMMENT ON COLUMN ACCION.IDESTADO IS 'Identificador del estado de la acción.';
COMMENT ON COLUMN ACCION.IDRIESGO IS 'Identificador del riesgo asociado.';

ALTER TABLE ACCION  
MODIFY (IDESTADO DEFAULT 1 );

/**
************************* Tabla Estado Accion *************************
*/
CREATE TABLE ESTADO_ACCION (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(50) NOT NULL, 
 CONSTRAINT PK_ESTADO_ACCION_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX ESTADO_ACCION_PK ON ESTADO_ACCION (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE ESTADO_ACCION
ADD CONSTRAINT UQ_ESTADO_ACCION_NOMBRE UNIQUE (NOMBRE)
ENABLE;
COMMENT ON COLUMN ESTADO_ACCION.ID IS 'Identificador del estado de la acción.';
COMMENT ON COLUMN ESTADO_ACCION.NOMBRE IS 'Nombre del estado.';

/**
************************* Tabla Estado Accion Historico *************************
*/
CREATE TABLE ACCION_ESTADO_HISTORICO (
 IDACCION NUMBER NOT NULL, 
 IDESTADOACTUAL NUMBER NOT NULL, 
 FECHAESTADO DATE NOT NULL, 
 IDESTADOANTERIOR NUMBER, 
 FECHAESTADOANTERIOR DATE,
 CONSTRAINT PK_ACCION_ESTADO_HISTORICO PRIMARY KEY (IDACCION, IDESTADOACTUAL, FECHAESTADO) 
 USING INDEX (CREATE UNIQUE INDEX ACCION_ESTADO_HISTORICO_PK ON ACCION_ESTADO_HISTORICO (IDACCION ASC, IDESTADOACTUAL ASC, FECHAESTADO ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN ACCION_ESTADO_HISTORICO.IDACCION IS 'Identificador de la acción.';
COMMENT ON COLUMN ACCION_ESTADO_HISTORICO.IDESTADOACTUAL IS 'Identificador del estado de la acción.';
COMMENT ON COLUMN ACCION_ESTADO_HISTORICO.FECHAESTADO IS 'Fecha de inserción del registro.';
COMMENT ON COLUMN ACCION_ESTADO_HISTORICO.IDESTADOANTERIOR IS 'Identificador del estado anterior de la acción.';
COMMENT ON COLUMN ACCION_ESTADO_HISTORICO.FECHAESTADOANTERIOR IS 'Fecha de inserción del registro del estado anterior';

/**
************************* Tabla Equipo Origen *************************
*/
CREATE TABLE EQUIPO_ORIGEN (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(50) NOT NULL, 
 CONSTRAINT PK_EQUIPO_ORIGEN_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX EQUIPO_ORIGEN_PK ON EQUIPO_ORIGEN (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE EQUIPO_ORIGEN
ADD CONSTRAINT UQ_EQUIPO_ORIGEN_NOMBRE UNIQUE (NOMBRE)
ENABLE;
COMMENT ON COLUMN EQUIPO_ORIGEN.ID IS 'Identificador del equipo de origen.';
COMMENT ON COLUMN EQUIPO_ORIGEN.NOMBRE IS 'Nombre del equipo de origen.';

/**
************************* Tabla Categoria *************************
*/
CREATE TABLE CATEGORIA (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(100) NOT NULL, 
 FECHA_CREACION DATE NOT NULL, 
 IDCREADOR NUMBER NOT NULL, 
 IDIMPORTANCIA NUMBER NOT NULL, 
 CONSTRAINT PK_CATEGORIA_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX CATEGORIA_PK ON CATEGORIA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE CATEGORIA
ADD CONSTRAINT UQ_CATEGORIA_NOMBRE UNIQUE (NOMBRE)
ENABLE;
COMMENT ON COLUMN CATEGORIA.ID IS 'Identificador de la categoría.';
COMMENT ON COLUMN CATEGORIA.NOMBRE IS 'Nombre de la categoría.';
COMMENT ON COLUMN CATEGORIA.FECHA_CREACION IS 'Fecha de creación de la categoría.';
COMMENT ON COLUMN CATEGORIA.IDCREADOR IS 'Identificador de la persona que crea la categoría.';
COMMENT ON COLUMN CATEGORIA.IDIMPORTANCIA IS 'Identificador de la importancia de la categoría.';

/**
************************* Tabla Importancia *************************
*/
CREATE TABLE IMPORTANCIA (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(50) NOT NULL, 
 VALOR SMALLINT NOT NULL, 
 CONSTRAINT PK_IMPORTANCIA_ID PRIMARY KEY (ID)
 USING INDEX (CREATE UNIQUE INDEX IMPORTANCIA_PK ON IMPORTANCIA (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE IMPORTANCIA
ADD CONSTRAINT UQ_IMPORTANCIA_NOMBRE_VALOR UNIQUE (NOMBRE, VALOR)
ENABLE;
COMMENT ON COLUMN IMPORTANCIA.ID IS 'Identificador de la importancia.';
COMMENT ON COLUMN IMPORTANCIA.NOMBRE IS 'Nombre de la importancia.';
COMMENT ON COLUMN IMPORTANCIA.VALOR IS 'Valor de la importancia.';

/**
************************* Tabla Criticidad *************************
*/
CREATE TABLE CRITICIDAD (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(50) NOT NULL, 
 VALOR SMALLINT NOT NULL, 
 CONSTRAINT PK_CRITICIDAD_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX CRITICIDAD_PK ON CRITICIDAD (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE CRITICIDAD
ADD CONSTRAINT UQ_CRITICIDAD_NOMBRE_VALOR UNIQUE (NOMBRE, VALOR)
ENABLE;
COMMENT ON COLUMN CRITICIDAD.ID IS 'Identificador de la criticidad.';
COMMENT ON COLUMN CRITICIDAD.NOMBRE IS 'Nombre de la criticidad.';
COMMENT ON COLUMN CRITICIDAD.VALOR IS 'Valor de la criticidad.';

/**
************************* Tabla Estado Riesgo *************************
*/
CREATE TABLE ESTADO_RIESGO (
 ID NUMBER NOT NULL, 
 NOMBRE VARCHAR2(50) NOT NULL, 
 CONSTRAINT PK_ESTADO_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX ESTADO_RIESGO_PK ON ESTADO_RIESGO (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;

ALTER TABLE ESTADO_RIESGO
ADD CONSTRAINT UQ_ESTADO_NOMBRE_VALOR UNIQUE (NOMBRE)
ENABLE;
COMMENT ON COLUMN ESTADO_RIESGO.ID IS 'Identificador del estado.';
COMMENT ON COLUMN ESTADO_RIESGO.NOMBRE IS 'Nombre del estado.';

/**
************************* Tabla Riesgo *************************
*/
CREATE TABLE RIESGO (
 ID NUMBER NOT NULL, 
 DESCRIPCION VARCHAR2(500) NOT NULL, 
 FECHA_CREACION DATE NOT NULL, 
 IMPACTO NUMBER NOT NULL, 
 IDCATEGORIA NUMBER NOT NULL, 
 IDCRITICIDAD NUMBER NOT NULL, 
 IDORIGEN NUMBER NOT NULL, 
 IDRESPONSABLE NUMBER NOT NULL, 
 IDESTADORIESGO NUMBER NOT NULL, 
 CONSTRAINT PK_RIESGO_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX RIESGO_PK ON RIESGO (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN RIESGO.ID IS 'Identificador del riesgo.';
COMMENT ON COLUMN RIESGO.DESCRIPCION IS 'Descripción del riesgo.';
COMMENT ON COLUMN RIESGO.FECHA_CREACION IS 'Fecha de creación.';
COMMENT ON COLUMN RIESGO.IMPACTO IS 'Impacto del riesgo (producto de criticidad por importancia).';
COMMENT ON COLUMN RIESGO.IDCATEGORIA IS 'Identificador de la categoría correspondiente al riesgo.';
COMMENT ON COLUMN RIESGO.IDCRITICIDAD IS 'Identificador de la criticidad correspondiente al riesgo.';
COMMENT ON COLUMN RIESGO.IDORIGEN IS 'Identificador del equipo de origen que determino el riesgo.';
COMMENT ON COLUMN RIESGO.IDRESPONSABLE IS 'Identificador de la persona responsable del riesgo.';
COMMENT ON COLUMN RIESGO.IDESTADORIESGO IS 'Identificador del estado del riesgo.';

ALTER TABLE RIESGO  
MODIFY (IDESTADORIESGO DEFAULT 1 );

/**
************************* Tabla Log *************************
*/
CREATE TABLE LOGGER (
 ID NUMBER NOT NULL, 
 NOMBRE_PROCEDIMIENTO VARCHAR2(128) NOT NULL, 
 ENTRADA VARCHAR2(1000) NOT NULL, 
 RSP VARCHAR2(50) NOT NULL, 
 OBSERVACIONES VARCHAR2(2000), 
 FECHA_ENTRADA DATE NOT NULL, 
 CONSTRAINT PK_LOGGER_ID PRIMARY KEY (ID) 
 USING INDEX (CREATE UNIQUE INDEX LOGGER_PK ON LOGGER (ID ASC))
 ENABLE 
) 
TABLESPACE TBS_TFG_DATOS;
COMMENT ON COLUMN LOGGER.ID IS 'Identificador del registro.';
COMMENT ON COLUMN LOGGER.NOMBRE_PROCEDIMIENTO IS 'Nombre del procedimiento que se registra.';
COMMENT ON COLUMN LOGGER.ENTRADA IS 'Parámetros de entrada con los que se ejecuta el procedimiento.';
COMMENT ON COLUMN LOGGER.RSP IS 'Valor de retorno del procedimiento.';
COMMENT ON COLUMN LOGGER.OBSERVACIONES IS 'Observaciones adicionales.';
COMMENT ON COLUMN LOGGER.FECHA_ENTRADA IS 'Fecha en la que se ejecutó el procedimiento.';

/**
************************* Tabla Repositorio estadístico general *************************
*/
CREATE TABLE REP_GENERAL (
 DEPTMASRIESGOSEQUIPOEXTERNO VARCHAR2(50),
 NUMRIESGCIBERAUTOEVAACTANT NUMBER, 
 ANHOMASACCIONIMPLEMRIESGMITIG NUMBER, 
 TOP3NOMBREACCIONESMAXTIEMPOABIERT VARCHAR2(1000), 
 PORCENTAJERIESGOSABIERTOSIMPACTOMENOR4 DECIMAL(5,2), 
 PERSONAMASACCIONESABIERTASASIGNADAS VARCHAR2(200), 
 NUMERORIESGOSCATEGORIA1 NUMBER 
) 
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN REP_GENERAL.DEPTMASRIESGOSEQUIPOEXTERNO IS 'Departamento con mayor número de riesgos detectados por auditorías externas.';
COMMENT ON COLUMN REP_GENERAL.NUMRIESGCIBERAUTOEVAACTANT IS 'Número de riesgos de ciberseguridad detectados por autoevaluación.';
COMMENT ON COLUMN REP_GENERAL.ANHOMASACCIONIMPLEMRIESGMITIG IS 'Año con mayor número de acciones que al final de año estaban en estado implementada con el riesgo mitigado.';
COMMENT ON COLUMN REP_GENERAL.TOP3NOMBREACCIONESMAXTIEMPOABIERT IS 'Top 3 de acciones que estuvieron más tiempo abiertas.';
COMMENT ON COLUMN REP_GENERAL.PORCENTAJERIESGOSABIERTOSIMPACTOMENOR4 IS 'En el momento de ejecutar la consulta, porcentaje de riesgos no corregidos o mitigados con un impacto menor a 4';
COMMENT ON COLUMN REP_GENERAL.PERSONAMASACCIONESABIERTASASIGNADAS IS 'En el momento de ejecutar la consulta, persona de la empresa que tiene más acciones abiertas (en estado de en curso o de definida) asignadas';
COMMENT ON COLUMN REP_GENERAL.NUMERORIESGOSCATEGORIA1 IS 'En el momento de ejecutar la consulta, número de riesgos de categoría 1 (en cualquier estado)';

/**
************************* Tabla Repositorio estadístico por año *************************
*/
CREATE TABLE REP_EST_ANHO_RIESGO (
 ANHO NUMBER NOT NULL, 
 NUMACCIONESESTADOCURSO NUMBER, 
 NUMRIESGOSIMPACTO1 NUMBER, 
 NUMRIESGOSESTADOABIERTO NUMBER, 
 DIFRIESGOSIMPACTO1INTEREXTER NUMBER, 
 COSTEMEDIOAUDITORIASEXTERNAS DECIMAL(10,2), 
 PORCENTAJEACCIONESDESCARTADAS DECIMAL(5,2), 
 NUMEROMEDIOMUESTREOS DECIMAL(10,2), 
 CONSTRAINT PK_REP_EST_ANHO_RIESGO_ANHO PRIMARY KEY (ANHO) 
 USING INDEX (CREATE UNIQUE INDEX REP_EST_ANHO_RIESGO_PK ON REP_EST_ANHO_RIESGO (ANHO ASC))
 ENABLE 
)
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.ANHO IS 'Año al que pertenecen los datos.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.NUMACCIONESESTADOCURSO IS 'Número de acciones en estado en curso.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.NUMRIESGOSIMPACTO1 IS 'Número de riesgos con impacto 1.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.NUMRIESGOSESTADOABIERTO IS 'Número de riesgos en estado abierto.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.DIFRIESGOSIMPACTO1INTEREXTER IS 'Diferencia de riesgos de impacto 1 detectados entre el equipo interno y equipo externo.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.COSTEMEDIOAUDITORIASEXTERNAS IS 'Coste medio de las auditorías externas.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.PORCENTAJEACCIONESDESCARTADAS IS 'Porcentaje de acciones en estado descartada.';
COMMENT ON COLUMN REP_EST_ANHO_RIESGO.NUMEROMEDIOMUESTREOS IS 'Número medio de muestreos.';

/**
************************* Tabla Repositorio estadístico de riesgos *************************
*/
CREATE TABLE TFG_GPOSESMZ.REP_AUX_RIESGO (
  IMPACTO_MENOR_4 NUMBER,
  IMPACTO_MENOR_4_ABIERTO NUMBER,
  CATEGORIA_1 NUMBER)
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO.IMPACTO_MENOR_4 IS 'Número de riesgos con impacto menor que 4';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO.IMPACTO_MENOR_4_ABIERTO IS 'Número de riesgos abiertos con impacto menor que 4';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO.CATEGORIA_1 IS 'Número de riesgos de categoria 1';

/**
************************* Tabla Repositorio estadístico de riesgos por año *************************
*/
CREATE TABLE TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO (
  ANHO NUMBER,
  IMPACTO_1 NUMBER,
  ABIERTOS NUMBER,
  IMPACTO_1_INTERNA NUMBER,
  IMPACTO_1_EXTERNA NUMBER,
  CIBERSEGURIDAD_AUTOEVALUACION NUMBER)
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.ANHO IS 'Año de riesgo';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.IMPACTO_1 IS 'Número de riesgos impacto igual a 1';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.ABIERTOS IS 'Número de riesgos abiertos';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.IMPACTO_1_INTERNA IS 'Número de riesgos impacto igual a 1 de auditorias internas';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.IMPACTO_1_EXTERNA IS 'Número de riesgos impacto igual a 1 de auditorias externas';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_RIESGO_ANHO.CIBERSEGURIDAD_AUTOEVALUACION IS 'Número de riesgos de ciberseguridad detectados por autoevaluación.';

/**
************************* Tabla Repositorio estadístico de departamentos *************************
*/
CREATE TABLE TFG_GPOSESMZ.REP_AUX_DEPARTAMENTO (
  NOMBRE VARCHAR2(500),
  RIESGOS_AUDITORIAS_EXTERNAS NUMBER) 
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_DEPARTAMENTO.NOMBRE IS 'Nombre de departamento.';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_DEPARTAMENTO.RIESGOS_AUDITORIAS_EXTERNAS IS 'Número de riesgos detectados por auditorías externas.';

/**
************************* Tabla Repositorio estadístico de acciones *************************
*/
CREATE TABLE REP_AUX_ACCION_ANHO (
	ANHO NUMBER,
	ESTADO_CURSO NUMBER,
	TOTAL_ACCIONES NUMBER,
	DESCARTADAS NUMBER,
	ESTADO_RIESGO_MITIGADO NUMBER) 
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_ACCION_ANHO.ANHO IS 'Año de accion';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_ACCION_ANHO.ESTADO_CURSO IS 'Número de acciones de estado En curso.';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_ACCION_ANHO.TOTAL_ACCIONES IS 'Número total de acciones';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_ACCION_ANHO.DESCARTADAS IS 'Número total de acciones descartadas';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_ACCION_ANHO.ESTADO_RIESGO_MITIGADO IS 'Número total de acciones en estado Implementada con riesgo mitigado';

/**
************************* Tabla Repositorio estadístico de PERSONA *************************
*/
CREATE TABLE REP_AUX_PERSONA (
	NOMBRE VARCHAR2(200),
	ACCIONES_ABIERTAS_ASIGNADAS NUMBER) 
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_PERSONA.NOMBRE IS 'Nombre de la persona';
COMMENT ON COLUMN TFG_GPOSESMZ.REP_AUX_PERSONA.ACCIONES_ABIERTAS_ASIGNADAS IS 'Número de acciones en estado abiertas o en curso que la persona tiene asignadas.';

/**
************************* Tabla Repositorio estadístico de top 3 de acciones de cambio de estado desde abierta hasta implementada. Por año *************************
*/
CREATE TABLE REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO (
	ANHO NUMBER,
	IDACCION VARCHAR2(200),
	DIAS NUMBER) 
TABLESPACE TBS_TFG_REP_ESTADISTICO;
COMMENT ON COLUMN REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO.ANHO IS 'Año de acción';
COMMENT ON COLUMN REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO.IDACCION IS 'Identificador de la acción';
COMMENT ON COLUMN REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO.DIAS IS 'Número de días entre el cambio.';

/**
************************* Tabla temporal para guardar id y descripciones de acciones para usar en el respositorio estadístico *************************
*/
CREATE GLOBAL TEMPORARY TABLE TEMP_ACCION(
	ID NUMBER NOT NULL,
	DESCRIPCION VARCHAR2(200) NOT NULL
)
ON COMMIT PRESERVE ROWS;
/
