
alter session set current_schema = TFG_GPOSESMZ;

/**
************************* Triggers de secuencuencias *************************
*/
CREATE OR REPLACE TRIGGER T_ANT_INS_CAMPANHA
    BEFORE INSERT ON CAMPANHA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_CAMPANHA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_CAMPANHA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_AUDITORIA
    BEFORE INSERT ON AUDITORIA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_AUDITORIA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_AUDITORIA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_EMPRESA
    BEFORE INSERT ON EMPRESA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_EMPRESA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_EMPRESA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_EMPRESA_EXTERNA
    BEFORE INSERT ON EMPRESA_EXTERNA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_EMPRESA_EXTERNA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_EMPRESA_EXTERNA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_DEPARTAMENTO
    BEFORE INSERT ON DEPARTAMENTO
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_DEPARTAMENTO_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_DEPARTAMENTO;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_MUESTREO
    BEFORE INSERT ON MUESTREO
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_MUESTREO_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_MUESTREO;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_CONTROL
    BEFORE INSERT ON CONTROL
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_MUESTREO_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_CONTROL;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_PERSONA
    BEFORE INSERT ON PERSONA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_PERSONA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_PERSONA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_ACCION
    BEFORE INSERT ON ACCION
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_ACCION_ID.NEXTVAL INTO :NEW.id FROM DUAL;
	
	DELETE FROM TEMP_ACCION;
	
	INSERT INTO TEMP_ACCION(ID, DESCRIPCION)
		SELECT ID, DESCRIPCION FROM ACCION;
		
END T_ANT_INS_ACCION;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_ESTADO_ACCION
    BEFORE INSERT ON ESTADO_ACCION
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_ESTADO_ACCION_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_ESTADO_ACCION;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_ESTADO_RIESGO
    BEFORE INSERT ON ESTADO_RIESGO
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_ESTADO_RIESGO_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_ESTADO_RIESGO;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_ORIGEN
    BEFORE INSERT ON EQUIPO_ORIGEN
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_ORIGEN_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_ORIGEN;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_RIESGO
    BEFORE INSERT ON RIESGO
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_RIESGO_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_RIESGO;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_CATEGORIA
    BEFORE INSERT ON CATEGORIA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_CATEGORIA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_CATEGORIA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_IMPORTANCIA
    BEFORE INSERT ON IMPORTANCIA
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_IMPORTANCIA_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_IMPORTANCIA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_CRITICIDAD
    BEFORE INSERT ON CRITICIDAD
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_CRITICIDAD_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_CRITICIDAD;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_LOGGER
    BEFORE INSERT ON LOGGER
    FOR EACH ROW
	
	BEGIN
    SELECT SEQ_LOGGER_ID.NEXTVAL INTO :NEW.id FROM DUAL;
END T_ANT_INS_LOG;
/

/**
************************* Triggers de funcionalidad *************************
*/
CREATE OR REPLACE TRIGGER T_ANT_INS_ACT_PERSONA
    BEFORE INSERT OR UPDATE ON PERSONA
    FOR EACH ROW	    
    DECLARE
    EX_SOLO_UNA_EMPRESA EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_SOLO_UNA_EMPRESA, -20111);
    BEGIN
        IF (:NEW.idempresa is not null and :NEW.idempresaexterna is not null) THEN
            RAISE_APPLICATION_ERROR(-20111, 'Solo se debe informar un parametro entre idempresa o idempresaexterna.');
        ELSIF (:NEW.idempresa is null and :NEW.idempresaexterna is null) THEN        
            RAISE_APPLICATION_ERROR(-20111, 'Se debe informar un parametro entre idempresa o idempresaexterna.');
        END IF;
        
END T_ANT_INS_ACT_PERSONA;
/

CREATE OR REPLACE TRIGGER T_ANT_INS_ACT_RIESGO
    BEFORE INSERT OR UPDATE ON RIESGO
    FOR EACH ROW	
    
    DECLARE
        importanciaCategoriaValor NUMBER;
        criticidadValor NUMBER;
		
    BEGIN
        SELECT IMP.VALOR 
            INTO importanciaCategoriaValor
        FROM CATEGORIA CAT
            INNER JOIN IMPORTANCIA IMP ON CAT.IDIMPORTANCIA = IMP.ID
        WHERE CAT.ID = :NEW.IDCATEGORIA;
        
        SELECT VALOR
            INTO criticidadValor
        FROM CRITICIDAD CRI 
        WHERE CRI.ID = :NEW.IDCRITICIDAD;
        
        :NEW.IMPACTO := importanciaCategoriaValor * criticidadValor;
        
END T_ANT_INS_ACT_RIESGO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_RIESGO
    AFTER INSERT OR UPDATE ON RIESGO
    FOR EACH ROW
    
    DECLARE
        ID_ESTADO_RIESGO_ABIERTO NUMBER;
        ID_EQUIPO_INTERNO NUMBER;
        ID_EQUIPO_EXTERNO NUMBER;
        ID_EQUIPO_AUTOEVALUACION NUMBER;
        ID_CATEGORIA_CIBERSEGURIDAD NUMBER;
        impactoMenor4 NUMBER;
        impactoMenor4Abierto NUMBER;
        impacto1PorAnho NUMBER;
        abiertosPorAnho NUMBER;
        impacto1Interna NUMBER;
        impacto1Externa NUMBER;
        categoria1 NUMBER;
        categoriaCiberseguridad NUMBER;
        NOMBREDEPARTAMENTOAUX VARCHAR2(50);
        NUMRIESGOSDEPARTAMENTOAUDITORIASEXTERNAS NUMBER;
        vAnho NUMBER;
        vAux NUMBER;
    BEGIN
    
        SELECT ID INTO ID_ESTADO_RIESGO_ABIERTO FROM ESTADO_RIESGO WHERE NOMBRE = 'Abierto';
        SELECT ID INTO ID_EQUIPO_INTERNO FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Auditoria interna';
        SELECT ID INTO ID_EQUIPO_EXTERNO FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Auditoria externa';
        SELECT ID INTO ID_EQUIPO_AUTOEVALUACION FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Autoevaluación';
        SELECT ID INTO ID_CATEGORIA_CIBERSEGURIDAD FROM CATEGORIA WHERE NOMBRE = 'Ciberseguridad';                
        vAnho := extract( year from :NEW.fecha_creacion);
        
        --Valores Actuales
        BEGIN
            SELECT IMPACTO_MENOR_4, IMPACTO_MENOR_4_ABIERTO, CATEGORIA_1
                INTO impactoMenor4, impactoMenor4Abierto, categoria1
            FROM REP_AUX_RIESGO;
        EXCEPTION WHEN NO_DATA_FOUND THEN
            impactoMenor4 := 0;
            impactoMenor4Abierto := 0;
            categoria1 := 0;
        END;
        
        BEGIN
            select IMPACTO_1, ABIERTOS, IMPACTO_1_INTERNA, IMPACTO_1_EXTERNA, CIBERSEGURIDAD_AUTOEVALUACION
                INTO impacto1PorAnho, abiertosPorAnho, impacto1Interna, impacto1Externa, categoriaCiberseguridad
            FROM REP_AUX_RIESGO_ANHO
            WHERE ANHO = vAnho;     
        EXCEPTION WHEN NO_DATA_FOUND THEN
            impacto1PorAnho := 0;
            abiertosPorAnho := 0;
            impacto1Interna := 0;
            impacto1Externa := 0;
            categoriaCiberseguridad := 0;
        END;
        -- En el momento de ejecutar la consulta, porcentaje de riesgos no corregidos o mitigados con un impacto menor a 4
        IF INSERTING THEN 
            IF :NEW.IMPACTO < 4 THEN
                impactoMenor4 := impactoMenor4 + 1;
            END IF;
            
            IF :NEW.IMPACTO < 4 AND :NEW.IDESTADORIESGO = ID_ESTADO_RIESGO_ABIERTO THEN
                impactoMenor4Abierto := impactoMenor4Abierto + 1;
            END IF;
        ELSE 
            -- Si cambia el impacto
            IF (:NEW.IMPACTO <> :OLD.IMPACTO) THEN
                IF :NEW.IMPACTO < 4 THEN
                    impactoMenor4 := impactoMenor4 + 1;
                ELSE 
                    impactoMenor4 := impactoMenor4 - 1;
                END IF;
            END IF;
            
            --Si cambia el estado
            IF (:NEW.IDESTADORIESGO <> :OLD.IDESTADORIESGO) THEN
                IF :NEW.IDESTADORIESGO = ID_ESTADO_RIESGO_ABIERTO THEN
                    impactoMenor4Abierto := impactoMenor4Abierto + 1;
                ELSE 
                    impactoMenor4Abierto := impactoMenor4Abierto - 1;
                END IF;
            END IF;
        END IF;
        
        -- Dado un año en concreto, número total de riesgos con un impacto de 1.
        IF INSERTING THEN 
            IF :NEW.IMPACTO = 1 THEN
                impacto1PorAnho := impacto1PorAnho + 1;
            END IF;
        ELSE 
            IF (:NEW.IMPACTO <> :OLD.IMPACTO) THEN
                IF :NEW.IMPACTO = 1 THEN
                    impacto1PorAnho := impacto1PorAnho + 1;
                ELSE
                    impacto1PorAnho := impacto1PorAnho - 1;
                END IF;
            END IF;
        END IF;
        
        -- En el último año finalizado, número de riesgos que actualmente aún estan en estado de abierto
         IF INSERTING THEN 
            IF :NEW.IDESTADORIESGO = ID_ESTADO_RIESGO_ABIERTO THEN
                abiertosPorAnho := abiertosPorAnho + 1;
            END IF;
        ELSE 
            IF (:NEW.IDESTADORIESGO <> :OLD.IDESTADORIESGO) THEN
                IF :NEW.IDESTADORIESGO = ID_ESTADO_RIESGO_ABIERTO THEN
                    abiertosPorAnho := abiertosPorAnho + 1;
                ELSE
                    abiertosPorAnho := abiertosPorAnho - 1;
                END IF;
            END IF;
        END IF;
        
        -- Dado un año en concreto, diferencia entre el número de riesgos de impacto 1 detectados por auditoría interna y externa
        IF INSERTING THEN 
            IF :NEW.IMPACTO = 1 THEN
                IF :NEW.IDORIGEN = ID_EQUIPO_INTERNO THEN
                    impacto1Interna := impacto1Interna + 1;
                END IF;
                IF :NEW.IDORIGEN = ID_EQUIPO_EXTERNO THEN
                    impacto1Externa := impacto1Externa + 1;
                END IF;
            END IF;
        ELSE 
            IF :NEW.IMPACTO <> :OLD.IMPACTO OR :NEW.IDORIGEN <> :OLD.IDORIGEN THEN
                IF :NEW.IMPACTO = 1 THEN
                    IF :NEW.IDORIGEN = ID_EQUIPO_INTERNO THEN
                        impacto1Interna := impacto1Interna + 1;
                    END IF;
                    IF :NEW.IDORIGEN = ID_EQUIPO_EXTERNO THEN
                        impacto1Externa := impacto1Externa + 1;
                    END IF;                    
                ELSE
                    IF :NEW.IDORIGEN = ID_EQUIPO_INTERNO THEN
                        impacto1Interna := impacto1Interna - 1;
                    END IF;
                    IF :NEW.IDORIGEN = ID_EQUIPO_EXTERNO THEN
                        impacto1Externa := impacto1Externa - 1;
                    END IF;                    
                END IF;
            END IF;            
        END IF;
        
        -- En el momento de ejecutar la consulta, número de riesgos de categoría 1 (en cualquier estado)
        IF INSERTING THEN 
            IF :NEW.IDCATEGORIA = 1 THEN
                categoria1 := categoria1 + 1;
            END IF;
        ELSE 
            IF (:NEW.IDCATEGORIA <> :OLD.IDCATEGORIA) THEN
                IF :NEW.IDCATEGORIA = 1 THEN
                    categoria1 := categoria1 + 1;
                ELSE
                    categoria1 := categoria1 - 1;
                END IF;
            END IF;
        END IF;
        
        -- Teniendo en cuenta el año en curso y el anterior, número de riesgos de ciberseguridad detectados por autoevaluación de los departamentos.
        IF INSERTING THEN 
        
            IF :NEW.IDCATEGORIA = ID_CATEGORIA_CIBERSEGURIDAD  
                AND :NEW.IDORIGEN = ID_EQUIPO_AUTOEVALUACION
                THEN
                categoriaCiberseguridad := categoriaCiberseguridad + 1;
            END IF;
        ELSE 
            IF (:NEW.IDCATEGORIA <> :OLD.IDCATEGORIA OR :NEW.IDORIGEN <> :OLD.IDORIGEN)
                AND :NEW.IDORIGEN = ID_EQUIPO_AUTOEVALUACION
                THEN
                IF :NEW.IDCATEGORIA = ID_CATEGORIA_CIBERSEGURIDAD THEN
                    categoriaCiberseguridad := categoriaCiberseguridad + 1;
                ELSE
                    categoriaCiberseguridad := categoriaCiberseguridad - 1;
                END IF;
            END IF;
        END IF;
        
        -- Con todos los datos disponibles en la BD, departamento con un número mayor de riesgos detectados por auditorías externas.
        INSERT INTO REP_AUX_DEPARTAMENTO (NOMBRE, RIESGOS_AUDITORIAS_EXTERNAS)
            SELECT NOMBRE, 0 FROM DEPARTAMENTO D WHERE D.NOMBRE NOT IN (select NOMBRE from REP_AUX_DEPARTAMENTO);
        
        IF INSERTING THEN
            IF :NEW.IDORIGEN = ID_EQUIPO_EXTERNO THEN
                SELECT D.NOMBRE 
                    INTO nombreDepartamentoAux
                FROM PERSONA P
                    INNER JOIN DEPARTAMENTO D ON P.IDDEPARTAMENTO = D.ID
                WHERE P.ID = :NEW.IDRESPONSABLE;
                
                SELECT RIESGOS_AUDITORIAS_EXTERNAS
                    INTO numRiesgosDepartamentoAuditoriasExternas
                FROM REP_AUX_DEPARTAMENTO
                WHERE NOMBRE = nombreDepartamentoAux;
                
                UPDATE REP_AUX_DEPARTAMENTO
                SET RIESGOS_AUDITORIAS_EXTERNAS = numRiesgosDepartamentoAuditoriasExternas + 1 
                WHERE NOMBRE = nombreDepartamentoAux;
            END IF;
        ELSE
        
            
            IF (:NEW.IDORIGEN <> :OLD.IDORIGEN OR :NEW.IDRESPONSABLE <> :OLD.IDRESPONSABLE) THEN
                
                IF :NEW.IDORIGEN = ID_EQUIPO_EXTERNO THEN
                
                    SELECT D.NOMBRE 
                        INTO nombreDepartamentoAux
                    FROM PERSONA P
                        INNER JOIN DEPARTAMENTO D ON P.IDDEPARTAMENTO = D.ID
                    WHERE P.ID = :NEW.IDRESPONSABLE;
                    
                    SELECT RIESGOS_AUDITORIAS_EXTERNAS
                        INTO numRiesgosDepartamentoAuditoriasExternas
                    FROM REP_AUX_DEPARTAMENTO
                    WHERE NOMBRE = nombreDepartamentoAux;
                    
                    UPDATE REP_AUX_DEPARTAMENTO
                    SET RIESGOS_AUDITORIAS_EXTERNAS = numRiesgosDepartamentoAuditoriasExternas + 1
                    WHERE NOMBRE = nombreDepartamentoAux;
                
                    IF :NEW.IDRESPONSABLE <> :OLD.IDRESPONSABLE THEN
                    
                        SELECT D.NOMBRE 
                            INTO nombreDepartamentoAux
                        FROM PERSONA P
                            INNER JOIN DEPARTAMENTO D ON P.IDDEPARTAMENTO = D.ID
                        WHERE P.ID = :OLD.IDRESPONSABLE;
                        
                        SELECT RIESGOS_AUDITORIAS_EXTERNAS
                            INTO numRiesgosDepartamentoAuditoriasExternas
                        FROM REP_AUX_DEPARTAMENTO
                        WHERE NOMBRE = nombreDepartamentoAux;
                        
                        UPDATE REP_AUX_DEPARTAMENTO
                        SET RIESGOS_AUDITORIAS_EXTERNAS = numRiesgosDepartamentoAuditoriasExternas - 1
                        WHERE NOMBRE = nombreDepartamentoAux;
                    END IF;
                    
				ELSE
					SELECT D.NOMBRE 
                        INTO nombreDepartamentoAux
                    FROM PERSONA P
                        INNER JOIN DEPARTAMENTO D ON P.IDDEPARTAMENTO = D.ID
                    WHERE P.ID = :OLD.IDRESPONSABLE;
                    
                    SELECT RIESGOS_AUDITORIAS_EXTERNAS
                        INTO numRiesgosDepartamentoAuditoriasExternas
                    FROM REP_AUX_DEPARTAMENTO
                    WHERE NOMBRE = nombreDepartamentoAux;
                    
                    UPDATE REP_AUX_DEPARTAMENTO
                    SET RIESGOS_AUDITORIAS_EXTERNAS = numRiesgosDepartamentoAuditoriasExternas - 1
                    WHERE NOMBRE = nombreDepartamentoAux;

                END IF;
            END IF;
        END IF;
        
        -- Actualizamos repositorio estadistico
        SELECT COUNT(*) INTO vAux FROM REP_AUX_RIESGO;
        IF vAux = 0 THEN
            INSERT INTO REP_AUX_RIESGO (IMPACTO_MENOR_4_ABIERTO, IMPACTO_MENOR_4, CATEGORIA_1) 
				VALUES(impactoMenor4Abierto, impactoMenor4, categoria1);
		ELSE
			UPDATE REP_AUX_RIESGO 
			SET IMPACTO_MENOR_4_ABIERTO = impactoMenor4Abierto,
				IMPACTO_MENOR_4 = impactoMenor4,
				CATEGORIA_1 = categoria1;
        END IF;
		                
		MERGE INTO REP_AUX_RIESGO_ANHO repAuxRiesgoAnho using dual on (repAuxRiesgoAnho.anho = vAnho)
			WHEN NOT MATCHED THEN 
				INSERT (ANHO, IMPACTO_1, ABIERTOS, IMPACTO_1_INTERNA, IMPACTO_1_EXTERNA, CIBERSEGURIDAD_AUTOEVALUACION)
					VALUES (vAnho, impacto1PorAnho, abiertosPorAnho, impacto1Interna, impacto1Externa, categoriaCiberseguridad)
			WHEN MATCHED THEN
				UPDATE 
				SET IMPACTO_1 = impacto1PorAnho,
					ABIERTOS = abiertosPorAnho,
					IMPACTO_1_INTERNA = impacto1Interna, 
					IMPACTO_1_EXTERNA = impacto1Externa,
					CIBERSEGURIDAD_AUTOEVALUACION = categoriaCiberseguridad
				WHERE ANHO = vAnho;
		
END T_DES_INS_ACT_RIESGO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_ACCION
    AFTER INSERT OR UPDATE ON ACCION
	FOR EACH ROW
	
	DECLARE
        vAnho NUMBER;
        vAux NUMBER;
        ACCION_ESTADO_CURSO NUMBER;
        ACCION_ESTADO_DEFINIDA NUMBER;
        ACCION_ESTADO_DESCARTADA NUMBER;
        ACCION_ESTADO_IMPL_MITIGA NUMBER;
        ACCION_ESTADO_IMPL_CORREGIDO NUMBER;
        numAccionesEstadoCurso NUMBER;
        totalAcciones NUMBER;
        totalAccionesDescartadas NUMBER;
        totalAccionesEstaMitigado NUMBER;
        nombrePersona VARCHAR2(200);
		vIdEstadoAnterior NUMBER;
        vFechaEstadoAnterior DATE;
        
	BEGIN
		vAnho := extract( year from :NEW.fecha_creacion);
        SELECT ID INTO ACCION_ESTADO_DEFINIDA FROM ESTADO_ACCION WHERE NOMBRE = 'Definida';
        SELECT ID INTO ACCION_ESTADO_CURSO FROM ESTADO_ACCION WHERE NOMBRE = 'En curso';        
        SELECT ID INTO ACCION_ESTADO_IMPL_CORREGIDO FROM ESTADO_ACCION WHERE NOMBRE = 'Implementada con el riesgo corregido';
        SELECT ID INTO ACCION_ESTADO_IMPL_MITIGA FROM ESTADO_ACCION WHERE NOMBRE = 'Implementada con el riesgo mitigado';
        SELECT ID INTO ACCION_ESTADO_DESCARTADA FROM ESTADO_ACCION WHERE NOMBRE = 'Descartada';

		MERGE INTO TEMP_ACCION TA USING DUAL ON (ID = :NEW.ID)
		WHEN NOT MATCHED THEN
			INSERT (ID, DESCRIPCION)
			VALUES (:NEW.ID, :NEW.DESCRIPCION);

        --Valores Actuales
		BEGIN
			SELECT ESTADO_CURSO, TOTAL_ACCIONES, DESCARTADAS, ESTADO_RIESGO_MITIGADO
				INTO numAccionesEstadoCurso, totalAcciones, totalAccionesDescartadas, totalAccionesEstaMitigado
			FROM REP_AUX_ACCION_ANHO
			WHERE ANHO = vAnho;
		EXCEPTION WHEN NO_DATA_FOUND THEN
			numAccionesEstadoCurso := 0;
			totalAcciones := 0;
			totalAccionesDescartadas := 0;
            totalAccionesEstaMitigado := 0;
		END;
        
        -- En el año actual, número total de acciones en estado de en curso
        IF INSERTING THEN 
            IF :NEW.IDESTADO = ACCION_ESTADO_CURSO THEN
                numAccionesEstadoCurso := numAccionesEstadoCurso + 1;
            END IF;
        ELSE
            IF :NEW.IDESTADO <> :OLD.IDESTADO THEN
                IF :NEW.IDESTADO = ACCION_ESTADO_CURSO THEN
                    numAccionesEstadoCurso := numAccionesEstadoCurso + 1;
                ELSIF :OLD.IDESTADO = ACCION_ESTADO_CURSO THEN
                    numAccionesEstadoCurso := numAccionesEstadoCurso - 1;
                END IF;
            END IF;
        END IF;
        
        -- Dado un año en concreto, porcentaje de acciones descartadas.
        IF INSERTING THEN         
            totalAcciones := totalAcciones + 1;
        
            IF :NEW.IDESTADO = ACCION_ESTADO_DESCARTADA THEN
                totalAccionesDescartadas := totalAccionesDescartadas + 1;
            END IF;
        ELSE
            IF :NEW.IDESTADO <> :OLD.IDESTADO THEN
                IF :NEW.IDESTADO = ACCION_ESTADO_DESCARTADA THEN
                    totalAccionesDescartadas := totalAccionesDescartadas + 1;
                ELSIF :OLD.IDESTADO = ACCION_ESTADO_DESCARTADA THEN
                    totalAccionesDescartadas := totalAccionesDescartadas - 1;
                END IF;
            END IF;
        END IF;
        
        -- Teniendo en cuenta todos los datos disponibles, año con un número mayor de acciones que al final del año en cuestión estaban en estado de implementada con el riesgo mitigado
        IF INSERTING THEN
            IF :NEW.IDESTADO = ACCION_ESTADO_IMPL_MITIGA THEN
                totalAccionesEstaMitigado := totalAccionesEstaMitigado + 1;
            END IF;
         ELSE
            IF :NEW.IDESTADO <> :OLD.IDESTADO THEN
                IF :NEW.IDESTADO = ACCION_ESTADO_IMPL_MITIGA THEN
                    totalAccionesEstaMitigado := totalAccionesEstaMitigado + 1;
                ELSIF :OLD.IDESTADO = ACCION_ESTADO_IMPL_MITIGA THEN
                    totalAccionesEstaMitigado := totalAccionesEstaMitigado - 1;
                END IF;
            END IF;
        END IF;
        
        -- En el momento de ejecutar la consulta, persona de la empresa que tiene más acciones abiertas (en estado de en curso o de definida) asignadas
        IF INSERTING THEN
            IF :NEW.IDESTADO = ACCION_ESTADO_DEFINIDA OR :NEW.IDESTADO = ACCION_ESTADO_CURSO THEN
                
                SELECT NOMBRE
                    INTO nombrePersona
                FROM PERSONA
                WHERE ID = :NEW.IDRESPONSABLE;
                
                MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)
                WHEN NOT MATCHED THEN 
                    INSERT (NOMBRE, ACCIONES_ABIERTAS_ASIGNADAS)
                        VALUES (nombrePersona, 1)
                WHEN MATCHED THEN
                    UPDATE
                    SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS + 1
                    WHERE NOMBRE = nombrePersona;
                
            END IF;
        ELSE
            -- Si los responsables son diferentes y estados son iguales y son abiertos y los anteriores no son corregidos mitigados o descartados
            IF :NEW.IDRESPONSABLE <> :OLD.IDRESPONSABLE 
                AND :NEW.IDESTADO = :OLD.IDESTADO 
                AND (:NEW.IDESTADO = ACCION_ESTADO_DEFINIDA OR :NEW.IDESTADO = ACCION_ESTADO_CURSO)
                THEN
                
                -- Restamos uno al antiguo responsable
                SELECT NOMBRE
                    INTO nombrePersona
                FROM PERSONA
                WHERE ID = :OLD.IDRESPONSABLE;
                
                UPDATE REP_AUX_PERSONA
                SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS - 1 
                WHERE NOMBRE = nombrePersona;
                
                -- Sumamos uno al nuevo responsable
                SELECT NOMBRE
                    INTO nombrePersona
                FROM PERSONA
                WHERE ID = :NEW.IDRESPONSABLE;
                
                MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)
                WHEN NOT MATCHED THEN 
                    INSERT (NOMBRE, ACCIONES_ABIERTAS_ASIGNADAS)
                        VALUES (nombrePersona, 1)
                WHEN MATCHED THEN
                    UPDATE
                    SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS + 1
                    WHERE NOMBRE = nombrePersona;
                    
            -- Si el responsable no cambia y los estados son diferentes
            ELSIF :NEW.IDRESPONSABLE = :OLD.IDRESPONSABLE 
                AND :NEW.IDESTADO <> :OLD.IDESTADO 
                THEN

                SELECT NOMBRE
                    INTO nombrePersona
                FROM PERSONA
                WHERE ID = :NEW.IDRESPONSABLE;
                                
                -- si el nuevo estado es abierto
                IF (:NEW.IDESTADO = ACCION_ESTADO_DEFINIDA OR :NEW.IDESTADO = ACCION_ESTADO_CURSO)
                    AND (:OLD.IDESTADO = ACCION_ESTADO_IMPL_CORREGIDO OR :OLD.IDESTADO = ACCION_ESTADO_IMPL_MITIGA OR :OLD.IDESTADO = ACCION_ESTADO_DESCARTADA)
                THEN
                    
                    MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)
                    WHEN NOT MATCHED THEN 
                        INSERT (NOMBRE, ACCIONES_ABIERTAS_ASIGNADAS)
                            VALUES (nombrePersona, 1)
                    WHEN MATCHED THEN
                        UPDATE
                        SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS + 1
                        WHERE NOMBRE = nombrePersona;
                        
                ELSIF (:OLD.IDESTADO = ACCION_ESTADO_DEFINIDA OR :OLD.IDESTADO = ACCION_ESTADO_CURSO)
                    AND (:NEW.IDESTADO = ACCION_ESTADO_IMPL_CORREGIDO OR :NEW.IDESTADO = ACCION_ESTADO_IMPL_MITIGA OR :NEW.IDESTADO = ACCION_ESTADO_DESCARTADA)
                    THEN
                    
                    UPDATE REP_AUX_PERSONA
                    SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS - 1 
                    WHERE NOMBRE = nombrePersona;
                END IF;
            -- Si el responsable cambia y el estado tambien
            ELSIF :NEW.IDRESPONSABLE <> :OLD.IDRESPONSABLE AND :NEW.IDESTADO <> :OLD.IDESTADO THEN
            
                IF (:NEW.IDESTADO = ACCION_ESTADO_DEFINIDA OR :NEW.IDESTADO = ACCION_ESTADO_CURSO)
                    AND (:OLD.IDESTADO = ACCION_ESTADO_IMPL_CORREGIDO OR :OLD.IDESTADO = ACCION_ESTADO_IMPL_MITIGA OR :OLD.IDESTADO = ACCION_ESTADO_DESCARTADA)
                THEN
                    SELECT NOMBRE
                    INTO nombrePersona
                    FROM PERSONA
                    WHERE ID = :NEW.IDRESPONSABLE;
                    
                    MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)
                    WHEN NOT MATCHED THEN 
                        INSERT (NOMBRE, ACCIONES_ABIERTAS_ASIGNADAS)
                            VALUES (nombrePersona, 1)
                    WHEN MATCHED THEN
                        UPDATE
                        SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS + 1
                        WHERE NOMBRE = nombrePersona;
                        
                    SELECT NOMBRE
                    INTO nombrePersona
                    FROM PERSONA
                    WHERE ID = :OLD.IDRESPONSABLE;
                    
                    MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)                    
                    WHEN MATCHED THEN
                        UPDATE
                        SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS - 1
                        WHERE NOMBRE = nombrePersona;
                END IF;
                
                IF (:OLD.IDESTADO = ACCION_ESTADO_DEFINIDA OR :OLD.IDESTADO = ACCION_ESTADO_CURSO)
                    AND (:NEW.IDESTADO = ACCION_ESTADO_IMPL_CORREGIDO OR :NEW.IDESTADO = ACCION_ESTADO_IMPL_MITIGA OR :NEW.IDESTADO = ACCION_ESTADO_DESCARTADA)
                THEN
                        
                    SELECT NOMBRE
                    INTO nombrePersona
                    FROM PERSONA
                    WHERE ID = :OLD.IDRESPONSABLE;
                    
                    MERGE INTO REP_AUX_PERSONA repAuxPersona using dual on (repAuxPersona.nombre = nombrePersona)                    
                    WHEN MATCHED THEN
                        UPDATE
                        SET ACCIONES_ABIERTAS_ASIGNADAS = ACCIONES_ABIERTAS_ASIGNADAS - 1
                        WHERE NOMBRE = nombrePersona;
                END IF;    
            END IF;
        END IF;
		
		-- Mantener histórico de estados de las acciones.
		IF INSERTING THEN 
			INSERT INTO ACCION_ESTADO_HISTORICO (IDACCION, IDESTADOACTUAL, FECHAESTADO)
				VALUES (:NEW.ID, :NEW.IDESTADO, SYSDATE);
				
		ELSIF (UPDATING AND :NEW.IDESTADO <> :OLD.IDESTADO) THEN 
		
			SELECT IDESTADOACTUAL, FECHAESTADO
				INTO vIdEstadoAnterior, vFechaEstadoAnterior
			FROM ACCION_ESTADO_HISTORICO
			WHERE IDACCION = :NEW.ID 
				AND IDESTADOACTUAL = :OLD.IDESTADO;
		
			INSERT INTO ACCION_ESTADO_HISTORICO (IDACCION, IDESTADOACTUAL, FECHAESTADO, IDESTADOANTERIOR, FECHAESTADOANTERIOR)
				VALUES (:NEW.ID, :NEW.IDESTADO, SYSDATE, vIdEstadoAnterior, vFechaEstadoAnterior);
		END IF;
		
        -- Actualizamos		
		MERGE INTO REP_AUX_ACCION_ANHO repAuxAccionAnho using dual on (repAuxAccionAnho.anho = vAnho)
            WHEN NOT MATCHED THEN 
                INSERT (ANHO, ESTADO_CURSO, TOTAL_ACCIONES, DESCARTADAS, ESTADO_RIESGO_MITIGADO)
					VALUES (vAnho, numAccionesEstadoCurso, totalAcciones, totalAccionesDescartadas, totalAccionesEstaMitigado)
            WHEN MATCHED THEN
                UPDATE 
				SET ESTADO_CURSO = numAccionesEstadoCurso,
					TOTAL_ACCIONES = totalAcciones,
					DESCARTADAS = totalAccionesDescartadas,
                    ESTADO_RIESGO_MITIGADO = totalAccionesEstaMitigado
				WHERE ANHO = vAnho;
		
END T_DES_INS_ACT_ACCION;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_AUDITORIA
    AFTER INSERT OR UPDATE ON AUDITORIA
    
    DECLARE
        anhoCurso NUMBER;
        numeroTotalAuditoriasCoste NUMBER;
        sumaTotalAuditoriasCoste NUMBER;
        costeMedio NUMBER(10,2);
        
	BEGIN
        select TO_NUMBER(to_char(sysdate, 'YYYY')) into anhoCurso from dual;
        
        SELECT COUNT(*)
            INTO numeroTotalAuditoriasCoste
        FROM AUDITORIA
        WHERE EXTRACT (YEAR FROM FECHA_INICIO) = anhoCurso
        AND COSTE <> 0;
        
        SELECT SUM(COSTE)
            INTO sumaTotalAuditoriasCoste
        FROM AUDITORIA
        WHERE EXTRACT (YEAR FROM FECHA_INICIO) = anhoCurso
        AND COSTE <> 0;
        
        BEGIN
            costeMedio := sumaTotalAuditoriasCoste / numeroTotalAuditoriasCoste;
        EXCEPTION WHEN OTHERS THEN
            costeMedio := NULL;
        END;
        
        MERGE INTO REP_EST_ANHO_RIESGO repRiesgo using dual on (repRiesgo.anho = anhoCurso)
            WHEN NOT MATCHED THEN 
                INSERT (ANHO, COSTEMEDIOAUDITORIASEXTERNAS) VALUES (anhoCurso, costeMedio)
            WHEN MATCHED THEN
                UPDATE SET COSTEMEDIOAUDITORIASEXTERNAS = costeMedio WHERE ANHO = anhoCurso;
        
	
END T_DES_INS_ACT_AUDITORIA;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_RIESGO
    AFTER INSERT OR UPDATE ON REP_AUX_RIESGO
    
    DECLARE        
        impactoMenor4 NUMBER;
        impactoMenor4Abierto NUMBER;
        categoria1 NUMBER;
        categoriaCiberseguridadAnhoActual NUMBER;
		categoriaCiberseguridadAnhoAnterior NUMBER;
		calculo DECIMAL(5,2) := 0 ;
        vAux NUMBER;
    BEGIN        
	
        SELECT IMPACTO_MENOR_4, IMPACTO_MENOR_4_ABIERTO, CATEGORIA_1
            INTO impactoMenor4, impactoMenor4Abierto, categoria1
        FROM REP_AUX_RIESGO;
		
		IF impactoMenor4 <> 0 THEN
			calculo := ROUND((impactoMenor4Abierto * 100) / impactoMenor4, 2);
		END IF;
		
		select TO_NUMBER(to_char(sysdate, 'YYYY')) INTO vAux FROM DUAL;
		
		BEGIN
			SELECT CIBERSEGURIDAD_AUTOEVALUACION
				INTO categoriaCiberseguridadAnhoActual
			FROM REP_AUX_RIESGO_ANHO
			WHERE ANHO = vAux;
		EXCEPTION WHEN NO_DATA_FOUND THEN
			categoriaCiberseguridadAnhoActual := 0;
		END;
		
		BEGIN
			SELECT CIBERSEGURIDAD_AUTOEVALUACION
				INTO categoriaCiberseguridadAnhoAnterior
			FROM REP_AUX_RIESGO_ANHO
			WHERE ANHO = vAux - 1;
		EXCEPTION WHEN NO_DATA_FOUND THEN
			categoriaCiberseguridadAnhoAnterior := 0;
		END;
		
        UPDATE REP_GENERAL
        SET PORCENTAJERIESGOSABIERTOSIMPACTOMENOR4 = calculo,
            NUMERORIESGOSCATEGORIA1 = categoria1,
            NUMRIESGCIBERAUTOEVAACTANT = (categoriaCiberseguridadAnhoActual + categoriaCiberseguridadAnhoAnterior);
      
END T_DES_INS_ACT_RIESGO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_RIESGO_ANHO
    AFTER INSERT OR UPDATE ON REP_AUX_RIESGO_ANHO
    
    DECLARE
        CURSOR c_auxRiesgosAnho IS
            SELECT ANHO, IMPACTO_1, ABIERTOS, IMPACTO_1_INTERNA, IMPACTO_1_EXTERNA
            FROM REP_AUX_RIESGO_ANHO;        
        
        DIFERENCIA_IMPACTO_1_INTERNOS_EXTERNOS NUMBER;
    BEGIN        
    
        -- Dado un año en concreto, número total de riesgos con un impacto de 1.
        FOR fila in c_auxRiesgosAnho
        LOOP
            DIFERENCIA_IMPACTO_1_INTERNOS_EXTERNOS := fila.IMPACTO_1_INTERNA - fila.IMPACTO_1_EXTERNA;
            
            MERGE INTO REP_EST_ANHO_RIESGO repRiesgo using dual on (repRiesgo.anho = fila.anho)
                WHEN NOT MATCHED THEN 
                    INSERT (ANHO, NUMRIESGOSIMPACTO1, NUMRIESGOSESTADOABIERTO, DIFRIESGOSIMPACTO1INTEREXTER)
                        VALUES (fila.anho, fila.IMPACTO_1, fila.ABIERTOS, DIFERENCIA_IMPACTO_1_INTERNOS_EXTERNOS)
                WHEN MATCHED THEN
                    UPDATE
                    SET NUMRIESGOSIMPACTO1 = fila.IMPACTO_1, 
                        NUMRIESGOSESTADOABIERTO = fila.ABIERTOS,
                        DIFRIESGOSIMPACTO1INTEREXTER = DIFERENCIA_IMPACTO_1_INTERNOS_EXTERNOS
                    WHERE ANHO = fila.anho;
        END LOOP;
        
     
END T_DES_INS_ACT_RIESGO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_DEPARTAMENTO
    AFTER INSERT OR UPDATE ON REP_AUX_DEPARTAMENTO
    
    DECLARE
		nombreDepartamento VARCHAR2(50);
		numero NUMBER;
    
    BEGIN
    
    SELECT NOMBRE, RIESGOS_AUDITORIAS_EXTERNAS
        INTO nombreDepartamento, numero
    FROM
        (SELECT * FROM REP_AUX_DEPARTAMENTO
        ORDER BY RIESGOS_AUDITORIAS_EXTERNAS DESC)
    WHERE ROWNUM = 1;
    
	IF numero = 0 THEN
		nombreDepartamento := NULL;
	END IF;
	
    UPDATE REP_GENERAL
        SET DEPTMASRIESGOSEQUIPOEXTERNO = nombreDepartamento;
     
END T_DES_INS_ACT_REP_AUX_DEPARTAMENTO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_ACCION_ANHO
    AFTER INSERT OR UPDATE ON REP_AUX_ACCION_ANHO
    
    DECLARE
		anhoActual NUMBER;
		CURSOR c_auxAccionAnho IS
            SELECT ANHO, ESTADO_CURSO, TOTAL_ACCIONES, DESCARTADAS, ESTADO_RIESGO_MITIGADO
            FROM REP_AUX_ACCION_ANHO;
        calculo DECIMAL(5,2) := 0;
		anhoAccionesImplRiesMitigado NUMBER;
		vAux NUMBER := 0;
		
    BEGIN
	
	select TO_NUMBER(to_char(sysdate, 'YYYY')) INTO anhoActual FROM DUAL;
    
    -- En el año actual, número total de acciones en estado de en curso
    FOR fila IN c_auxAccionAnho
    LOOP
		
		--Año con mayor numero de acciones que al final del año en cuestion estaban en estado ded implementada.
		-- al final de año, es decir, año finalizado
		IF fila.ANHO <> anhoActual THEN
			IF fila.ESTADO_RIESGO_MITIGADO > vAux THEN
				anhoAccionesImplRiesMitigado := fila.ANHO;
				vAux := fila.ESTADO_RIESGO_MITIGADO;
			END IF;
		END IF;
		
        --numAccionesDescartadadas * 100)/numAcciones
        IF fila.TOTAL_ACCIONES <> 0 THEN
            calculo := (fila.DESCARTADAS * 100) / fila.TOTAL_ACCIONES;
        END IF;
        
        MERGE INTO REP_EST_ANHO_RIESGO repRiesgo using dual on (repRiesgo.anho = fila.ANHO)
            WHEN NOT MATCHED THEN 
                INSERT (ANHO, NUMACCIONESESTADOCURSO, PORCENTAJEACCIONESDESCARTADAS)
                    VALUES (fila.ANHO, fila.ESTADO_CURSO, calculo)
            WHEN MATCHED THEN
                UPDATE 
                    SET NUMACCIONESESTADOCURSO = fila.ESTADO_CURSO,
                        PORCENTAJEACCIONESDESCARTADAS = calculo
                WHERE ANHO = fila.ANHO;
    END LOOP;
	
	UPDATE rep_general
	SET ANHOMASACCIONIMPLEMRIESGMITIG = anhoAccionesImplRiesMitigado;
     
END T_DES_INS_ACT_REP_AUX_ACCION_ANHO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_PERSONA
    AFTER INSERT OR UPDATE ON REP_AUX_PERSONA
	
	DECLARE
        nombrePersona VARCHAR2(200);
	BEGIN
        SELECT NOMBRE
            INTO nombrePersona
        FROM (
            SELECT NOMBRE FROM REP_AUX_PERSONA
            ORDER BY ACCIONES_ABIERTAS_ASIGNADAS DESC
        )
        WHERE ROWNUM = 1;

    
        UPDATE REP_GENERAL
        SET PERSONAMASACCIONESABIERTASASIGNADAS = nombrePersona;
	
END T_DES_INS_ACT_REP_AUX_PERSONA;
/	

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_ACCION_ESTADO_HISTORICO
    AFTER INSERT ON ACCION_ESTADO_HISTORICO
    FOR EACH ROW
    
    DECLARE
    ID_ESTADO_DEFINIDA NUMBER;
    ID_ESTADO_IMPLEMENTADA_CORREGIDO NUMBER;
    ID_ESTADO_IMPLEMENTADA_MITIGADO NUMBER;
    anhoC NUMBER;
    idAccionMinima VARCHAR2(200);
    numDiasAccionMinima NUMBER;
    vAux NUMBER;
    diasDeCambioEstado NUMBER;
    
    BEGIN
        SELECT ID INTO ID_ESTADO_DEFINIDA FROM ESTADO_ACCION WHERE NOMBRE = 'Definida';
        SELECT ID INTO ID_ESTADO_IMPLEMENTADA_CORREGIDO FROM ESTADO_ACCION WHERE NOMBRE = 'Implementada con el riesgo corregido';
        SELECT ID INTO ID_ESTADO_IMPLEMENTADA_MITIGADO FROM ESTADO_ACCION WHERE NOMBRE = 'Implementada con el riesgo mitigado';
        anhoC := EXTRACT (YEAR FROM :NEW.FECHAESTADO);
        
        BEGIN
            SELECT COUNT(*)
                INTO vAux
            FROM REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO
            WHERE ANHO = anhoC;
        EXCEPTION WHEN NO_DATA_FOUND THEN
            vAux := 0;
        END;
        
        IF :NEW.IDESTADOANTERIOR = ID_ESTADO_DEFINIDA AND (:NEW.IDESTADOACTUAL = ID_ESTADO_IMPLEMENTADA_CORREGIDO OR :NEW.IDESTADOACTUAL = ID_ESTADO_IMPLEMENTADA_MITIGADO) THEN
        
            diasDeCambioEstado := :NEW.FECHAESTADO - :NEW.FECHAESTADOANTERIOR;
            IF vAux < 3 THEN
                INSERT INTO REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO (ANHO, IDACCION, DIAS)
                    VALUES (anhoC, :NEW.IDACCION, diasDeCambioEstado);
            ELSE
                SELECT IDACCION, DIAS
                    INTO idAccionMinima, numDiasAccionMinima
                FROM (
                    SELECT IDACCION, DIAS
                    FROM REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO
                    WHERE ANHO = anhoC
                    ORDER BY DIAS
                )
                WHERE ROWNUM = 1;
                
                IF diasDeCambioEstado > numDiasAccionMinima THEN
                    UPDATE REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO
                    SET IDACCION = :NEW.IDACCION,
                        DIAS = diasDeCambioEstado
                    WHERE ANHO = anhoC AND IDACCION = idAccionMinima;
                END IF;
                
            END IF;
        END IF;
		
		
        
END T_DES_INS_ACT_ACCION_ESTADO_HISTORICO;
/

CREATE OR REPLACE TRIGGER T_DES_INS_ACT_REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO
    AFTER INSERT OR UPDATE ON REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO
	
	DECLARE
		anhoFinalizado NUMBER;
        top3 VARCHAR2(1000);
	BEGIN
		anhoFinalizado := EXTRACT (YEAR FROM SYSDATE) - 1;
		
		BEGIN
			SELECT LISTAGG(A.DESCRIPCION, ', ') 			
				INTO top3
				FROM REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO R
					INNER JOIN TEMP_ACCION A ON R.IDACCION = A.ID
				WHERE ANHO = anhoFinalizado;    
		EXCEPTION WHEN NO_DATA_FOUND THEN
			top3 := NULL;
		END;
		
		UPDATE REP_GENERAL SET TOP3NOMBREACCIONESMAXTIEMPOABIERT = top3;
	
END T_DES_INS_ACT_REP_AUX_TOP3_ACCION_ABIERTA_IMPLE_DIAS_ANHO;
/
