
alter session set current_schema = TFG_GPOSESMZ;

/**
************************* Logger *************************
*/
CREATE OR REPLACE PACKAGE PKG_UTILIDADES AS
    EX_PARAMETROS_OBLIGATORIOS EXCEPTION;
	EX_PK_NO_ENCONTRADA EXCEPTION;
    EX_FK_NO_ENCONTRADA EXCEPTION;
	EX_EXCEPCION_NO_CONTROLADA EXCEPTION;
    	
	TEXTO_EX_PARAMETROS_OBLIGATORIOS CONSTANT LOGGER.RSP%TYPE := 'Error: Existen parámetros que son obligatorios.';
	TEXTO_EX_PK_NO_ENCONTRADA CONSTANT LOGGER.RSP%TYPE := 'Error. No existe la clave primaria.';
	TEXTO_EX_FK_NO_ENCONTRADA CONSTANT LOGGER.RSP%TYPE := 'Error. No existe la clave foránea.';
	TEXTO_EX_EXCEPCION_NO_CONTROLADA CONSTANT LOGGER.RSP%TYPE := 'Error. Excepción no controlada.';
	
	TYPE ARRAY_STRING IS TABLE OF VARCHAR2(250) INDEX BY VARCHAR2(50);
    TYPE ARRAY_NUMBER IS TABLE OF NUMBER INDEX BY VARCHAR2(50);
    TYPE ARRAY_DATE IS TABLE OF DATE INDEX BY VARCHAR2(50);

    PROCEDURE addLog (
        p_nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE,
        p_entrada LOGGER.ENTRADA%TYPE,
		p_rsp LOGGER.RSP%TYPE,
        p_observaciones LOGGER.OBSERVACIONES%TYPE
    );
	
	PROCEDURE compruebaCamposObligatorios (
        obligatoriosString IN ARRAY_STRING,
        obligatoriosNumber IN ARRAY_NUMBER,
        obligatoriosDate IN ARRAY_DATE,
        vParamAnalizado OUT VARCHAR2
    );
	
	PROCEDURE compruebaExistenciaIDEnTabla(
        p_ID IN NUMBER,
		p_nombreTabla IN VARCHAR2,
		p_pk IN BOOLEAN
    );

END PKG_UTILIDADES;
/

CREATE OR REPLACE PACKAGE BODY PKG_UTILIDADES AS

    PROCEDURE addLog (
        p_nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE,
        p_entrada LOGGER.ENTRADA%TYPE,
		p_rsp LOGGER.RSP%TYPE,
        p_observaciones LOGGER.OBSERVACIONES%TYPE
    ) IS
    BEGIN
        INSERT INTO LOGGER
        (NOMBRE_PROCEDIMIENTO, ENTRADA, RSP, OBSERVACIONES, FECHA_ENTRADA)
        VALUES
        (p_nombreProcedimiento, p_entrada, p_rsp, p_observaciones, SYSDATE);
    END addLog;
	
	PROCEDURE compruebaCamposObligatorios (
        obligatoriosString IN ARRAY_STRING,
        obligatoriosNumber IN ARRAY_NUMBER,
        obligatoriosDate IN ARRAY_DATE,
        vParamAnalizado OUT VARCHAR2
    ) AS
        
    BEGIN
        
        vParamAnalizado := obligatoriosString.FIRST;
        WHILE (vParamAnalizado IS NOT NULL) LOOP
            IF obligatoriosString(vParamAnalizado) IS NULL THEN
				RETURN;
            END IF;            
            vParamAnalizado := obligatoriosString.NEXT(vParamAnalizado);
        END LOOP;
        
        vParamAnalizado := obligatoriosNumber.FIRST;
        WHILE (vParamAnalizado IS NOT NULL) LOOP
            IF obligatoriosNumber(vParamAnalizado) IS NULL THEN
                RETURN;
            END IF;            
            vParamAnalizado := obligatoriosNumber.NEXT(vParamAnalizado);
        END LOOP;
        
        vParamAnalizado := obligatoriosDate.FIRST;
        WHILE (vParamAnalizado IS NOT NULL) LOOP
            IF obligatoriosDate(vParamAnalizado) IS NULL THEN
                RETURN;
            END IF;            
            vParamAnalizado := obligatoriosDate.NEXT(vParamAnalizado);
        END LOOP;
        vParamAnalizado := null;
		
    END compruebaCamposObligatorios;
	
	PROCEDURE compruebaExistenciaIDEnTabla(
        p_ID IN NUMBER,
		p_nombreTabla IN VARCHAR2,
		p_pk IN BOOLEAN
    )
    AS
        vAux NUMBER;
        BEGIN
			IF p_ID IS NULL THEN
				RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
			END IF;
            EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ' || p_nombreTabla || ' WHERE ID = ' || p_ID
            INTO vAux;
            IF vAux = 0 THEN
                IF p_pk THEN
                    RAISE PKG_UTILIDADES.EX_PK_NO_ENCONTRADA;
                END IF;
                IF NOT p_pk THEN
                    RAISE PKG_UTILIDADES.EX_FK_NO_ENCONTRADA;
                END IF;
                
            END IF;
    END compruebaExistenciaIDEnTabla;

END PKG_UTILIDADES;
/

/**
************************* Campaña *************************
*/
CREATE OR REPLACE PACKAGE PKG_CAMPANHA AS

    PROCEDURE altaCampanha (
        p_nombre IN CAMPANHA.NOMBRE%TYPE,
        p_fechaInicio IN CAMPANHA.FECHA_INICIO%TYPE,
		p_fechaFin IN CAMPANHA.FECHA_FIN%TYPE,
        p_idLider IN CAMPANHA.IDLIDER%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaCampanha (
        p_idCampanha IN CAMPANHA.ID%TYPE,
        p_nombre IN CAMPANHA.NOMBRE%TYPE,
        p_fechaInicio IN CAMPANHA.FECHA_INICIO%TYPE,
		p_fechaFin IN CAMPANHA.FECHA_FIN%TYPE,
        p_idLider IN CAMPANHA.IDLIDER%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_CAMPANHA;
/

CREATE OR REPLACE PACKAGE BODY PKG_CAMPANHA AS

    PROCEDURE altaCampanha (
        p_nombre IN CAMPANHA.NOMBRE%TYPE,
        p_fechaInicio IN CAMPANHA.FECHA_INICIO%TYPE,
		p_fechaFin IN CAMPANHA.FECHA_FIN%TYPE,
        p_idLider IN CAMPANHA.IDLIDER%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CAMPANHA.altaCampanha';
        parametrosEntrada LOGGER.ENTRADA%TYPE := 'p_nombre = ' || p_nombre ||', p_fechaInicio = ' || p_fechaInicio ||', p_fechaFin = ' || p_fechaFin ||', p_idLider = ' || p_idLider ||'';
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosDate('p_fechaInicio') := p_fechaInicio;
            obligatoriosDate('p_fechaFin') := p_fechaFin;
            obligatoriosNumber('p_idLider') := p_idLider;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idLider';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idLider, 'PERSONA', FALSE);
            
            -- Insertamos
            INSERT INTO CAMPANHA (NOMBRE, FECHA_INICIO, FECHA_FIN, IDLIDER)
                VALUES (p_nombre, p_fechaInicio, p_fechaFin, p_idLider)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campaña creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaCampanha;

    PROCEDURE actualizaCampanha (
        p_idCampanha IN CAMPANHA.ID%TYPE,
        p_nombre IN CAMPANHA.NOMBRE%TYPE,
        p_fechaInicio IN CAMPANHA.FECHA_INICIO%TYPE,
		p_fechaFin IN CAMPANHA.FECHA_FIN%TYPE,
        p_idLider IN CAMPANHA.IDLIDER%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CAMPANHA.actualizaCampanha';
        parametrosEntrada LOGGER.ENTRADA%TYPE := 'p_idCampanha = ' || p_idCampanha || ', p_nombre = ' || p_nombre ||', p_fechaInicio = ' || p_fechaInicio ||', p_fechaFin = ' || p_fechaFin ||', p_idLider = ' || p_idLider ||'';
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
            
			-- Comprobacion de clave primaria
			vParamAnalizado := 'p_idCampanha';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCampanha, 'CAMPANHA', TRUE);
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosDate('p_fechaInicio') := p_fechaInicio;
            obligatoriosDate('p_fechaFin') := p_fechaFin;
            obligatoriosNumber('p_idLider') := p_idLider;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
			
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idLider';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idLider, 'PERSONA', FALSE);
            
            -- Actualizamos
            UPDATE CAMPANHA 
            SET NOMBRE = p_nombre,
                FECHA_INICIO = p_fechaInicio,
                FECHA_FIN = p_fechaFin,
                IDLIDER = p_idLider
            WHERE ID = p_idCampanha;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campaña actualizada con el id: ' || p_idCampanha);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la campanha con el id: ' || p_idCampanha);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaCampanha;
    
END PKG_CAMPANHA;
/

/**
************************* Auditoria *************************
*/
CREATE OR REPLACE PACKAGE PKG_AUDITORIA AS

    PROCEDURE altaAuditoria (
        p_descripcion IN AUDITORIA.DESCRIPCION%TYPE,
		p_coste IN AUDITORIA.COSTE%TYPE,
        p_idCampanha IN AUDITORIA.IDCAMPANHA%TYPE,
		p_idEmpresa IN AUDITORIA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN AUDITORIA.IDEMPRESAEXTERNA%TYPE,
		p_fechaInicio IN AUDITORIA.FECHA_INICIO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaAuditoria (
        p_idAuditoria IN AUDITORIA.ID%TYPE,
        p_descripcion IN AUDITORIA.DESCRIPCION%TYPE,
		p_coste IN AUDITORIA.COSTE%TYPE,
        p_idCampanha IN AUDITORIA.IDCAMPANHA%TYPE,
		p_idEmpresa IN AUDITORIA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN AUDITORIA.IDEMPRESAEXTERNA%TYPE,
		p_fechaInicio IN AUDITORIA.FECHA_INICIO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_AUDITORIA;
/

CREATE OR REPLACE PACKAGE BODY PKG_AUDITORIA AS

    PROCEDURE altaAuditoria (
        p_descripcion IN AUDITORIA.DESCRIPCION%TYPE,
		p_coste IN AUDITORIA.COSTE%TYPE,
        p_idCampanha IN AUDITORIA.IDCAMPANHA%TYPE,
		p_idEmpresa IN AUDITORIA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN AUDITORIA.IDEMPRESAEXTERNA%TYPE,
		p_fechaInicio IN AUDITORIA.FECHA_INICIO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_AUDITORIA.altaAuditoria';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_coste = ' || p_coste;
			parametrosEntrada := parametrosEntrada || ', p_idCampanha = ' || p_idCampanha;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresaExterna = ' || p_idEmpresaExterna;
			parametrosEntrada := parametrosEntrada || ', p_fechaInicio = ' || p_fechaInicio;
			
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosDate('p_fechaInicio') := p_fechaInicio;
            obligatoriosNumber('p_idCampanha') := p_idCampanha;
			obligatoriosNumber('p_idEmpresa') := p_idEmpresa;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCampanha';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCampanha, 'CAMPANHA', FALSE);
			vParamAnalizado := 'p_idEmpresa';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
			vParamAnalizado := 'p_idEmpresaExterna';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresaExterna, 'EMPRESA_EXTERNA', FALSE);
            
            -- Insertamos
            INSERT INTO AUDITORIA (DESCRIPCION, COSTE, IDCAMPANHA, IDEMPRESA, IDEMPRESAEXTERNA, FECHA_INICIO)
                VALUES (p_descripcion, p_coste, p_idCampanha, p_idEmpresa, p_idEmpresaExterna, p_fechaInicio)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Auditoria creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaAuditoria;

    PROCEDURE actualizaAuditoria (
        p_idAuditoria IN AUDITORIA.ID%TYPE,
        p_descripcion IN AUDITORIA.DESCRIPCION%TYPE,
		p_coste IN AUDITORIA.COSTE%TYPE,
        p_idCampanha IN AUDITORIA.IDCAMPANHA%TYPE,
		p_idEmpresa IN AUDITORIA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN AUDITORIA.IDEMPRESAEXTERNA%TYPE,
		p_fechaInicio IN AUDITORIA.FECHA_INICIO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_AUDITORIA.actualizaAuditoria';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idAuditoria = ' || p_idAuditoria;
			parametrosEntrada := parametrosEntrada || ', p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_coste = ' || p_coste;
			parametrosEntrada := parametrosEntrada || ', p_idCampanha = ' || p_idCampanha;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresaExterna = ' || p_idEmpresaExterna;
			parametrosEntrada := parametrosEntrada || ', p_fechaInicio = ' || p_fechaInicio;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idAuditoria';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idAuditoria, 'AUDITORIA', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosDate('p_fechaInicio') := p_fechaInicio;
			obligatoriosNumber('p_idAuditoria') := p_idAuditoria;
            obligatoriosNumber('p_idCampanha') := p_idCampanha;
			obligatoriosNumber('p_idEmpresa') := p_idEmpresa;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCampanha';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCampanha, 'CAMPANHA', FALSE);
			vParamAnalizado := 'p_idEmpresa';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
			vParamAnalizado := 'p_idEmpresaExterna';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresaExterna, 'EMPRESA_EXTERNA', FALSE);
            
            -- Actualizamos
            UPDATE AUDITORIA 
            SET DESCRIPCION = p_descripcion,
				COSTE = p_coste,
                IDCAMPANHA = p_idCampanha,
                IDEMPRESA = p_idEmpresa,
                IDEMPRESAEXTERNA = p_idEmpresaExterna,
                FECHA_INICIO = p_fechaInicio
            WHERE ID = p_idAuditoria;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Auditoria actualizada con el id: ' || p_idAuditoria);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la auditoria con el id: ' || p_idAuditoria);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaAuditoria;
    
END PKG_AUDITORIA;
/

/**
************************* Empresa *************************
*/
CREATE OR REPLACE PACKAGE PKG_EMPRESA AS

    PROCEDURE altaEmpresa (
        p_cif IN EMPRESA.CIF%TYPE,
		p_nombre IN EMPRESA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaEmpresa (
        p_idEmpresa IN EMPRESA.ID%TYPE,
        p_cif IN EMPRESA.CIF%TYPE,
		p_nombre IN EMPRESA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_EMPRESA;
/

CREATE OR REPLACE PACKAGE BODY PKG_EMPRESA AS

    PROCEDURE altaEmpresa (
        p_cif IN EMPRESA.CIF%TYPE,
		p_nombre IN EMPRESA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EMPRESA.altaEmpresa';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_cif = ' || p_cif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_cif') := p_cif;
			obligatoriosString('p_nombre') := p_nombre;
			PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO EMPRESA (CIF, NOMBRE)
                VALUES (p_cif, p_nombre)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Empresa creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaEmpresa;

    PROCEDURE actualizaEmpresa (
        p_idEmpresa IN EMPRESA.ID%TYPE,
        p_cif IN EMPRESA.CIF%TYPE,
		p_nombre IN EMPRESA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EMPRESA.actualizaEmpresa';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idEmpresa = ' || p_idEmpresa			;
			parametrosEntrada := parametrosEntrada || ', p_cif = ' || p_cif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idEmpresa';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', TRUE);
            
            -- Comprobacion de campos obligatorios			
            obligatoriosString('p_cif') := p_cif;
			obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_idEmpresa') := p_idEmpresa;
			PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE EMPRESA 
            SET CIF = p_cif,
				NOMBRE = p_nombre
            WHERE ID = p_idEmpresa;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Empresa actualizada con el id: ' || p_idEmpresa);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la empresa con el id: ' || p_idEmpresa);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaEmpresa;
    
END PKG_EMPRESA;
/

/**
************************* EmpresaExterna *************************
*/
CREATE OR REPLACE PACKAGE PKG_EMPRESA_EXTERNA AS

	EX_CIF_EMPRESA EXCEPTION;    
	TEXTO_EX_CIF_EMPRESA CONSTANT LOGGER.RSP%TYPE := 'Error: El CIF ya existe en la tabla de empresa.';

    PROCEDURE altaEmpresaExterna (
        p_cif IN EMPRESA_EXTERNA.CIF%TYPE,
		p_nombre IN EMPRESA_EXTERNA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaEmpresaExterna (
        p_idEmpresaExterna IN EMPRESA_EXTERNA.ID%TYPE,
        p_cif IN EMPRESA_EXTERNA.CIF%TYPE,
		p_nombre IN EMPRESA_EXTERNA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_EMPRESA_EXTERNA;
/

CREATE OR REPLACE PACKAGE BODY PKG_EMPRESA_EXTERNA AS

    PROCEDURE altaEmpresaExterna (
        p_cif IN EMPRESA_EXTERNA.CIF%TYPE,
		p_nombre IN EMPRESA_EXTERNA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EMPRESA_EXTERNA.altaEmpresaExterna';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_cif = ' || p_cif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_cif') := p_cif;
			obligatoriosString('p_nombre') := p_nombre;
			PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
			SELECT COUNT(*)
				INTO vAux
			FROM EMPRESA
			WHERE UPPER(CIF) = UPPER(p_cif);			
			IF vAUX <> 0 THEN
				RAISE PKG_EMPRESA_EXTERNA.EX_CIF_EMPRESA;
			END IF;
			
            -- Insertamos
            INSERT INTO EMPRESA_EXTERNA (CIF, NOMBRE)
                VALUES (p_cif, p_nombre)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Empresa externa creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
			
			WHEN PKG_EMPRESA_EXTERNA.EX_CIF_EMPRESA THEN
                rsp := PKG_EMPRESA_EXTERNA.TEXTO_EX_CIF_EMPRESA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'CIF Ya existente: ' || p_cif);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaEmpresaExterna;

    PROCEDURE actualizaEmpresaExterna (
        p_idEmpresaExterna IN EMPRESA_EXTERNA.ID%TYPE,
        p_cif IN EMPRESA_EXTERNA.CIF%TYPE,
		p_nombre IN EMPRESA_EXTERNA.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EMPRESA_EXTERNA.actualizaEmpresa';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idEmpresaExterna = ' || p_idEmpresaExterna			;
			parametrosEntrada := parametrosEntrada || ', p_cif = ' || p_cif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idEmpresaExterna';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresaExterna, 'EMPRESA_EXTERNA', TRUE);
            
            -- Comprobacion de campos obligatorios			
            obligatoriosString('p_cif') := p_cif;
			obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_idEmpresaExterna') := p_idEmpresaExterna;
			PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
			
			SELECT COUNT(*)
				INTO vAux
			FROM EMPRESA
			WHERE UPPER(CIF) = UPPER(p_cif);			
			IF vAUX <> 0 THEN
				RAISE PKG_EMPRESA_EXTERNA.EX_CIF_EMPRESA;
			END IF;        
		
            -- Actualizamos
            UPDATE EMPRESA_EXTERNA 
            SET CIF = p_cif,
				NOMBRE = p_nombre
            WHERE ID = p_idEmpresaExterna;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Empresa externa actualizada con el id: ' || p_idEmpresaExterna);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la empresa externa con el id: ' || p_idEmpresaExterna);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
			
			WHEN PKG_EMPRESA_EXTERNA.EX_CIF_EMPRESA THEN
                rsp := PKG_EMPRESA_EXTERNA.TEXTO_EX_CIF_EMPRESA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'CIF Ya existente: ' || p_cif);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaEmpresaExterna;
    
END PKG_EMPRESA_EXTERNA;
/

/**
************************* Departamento *************************
*/
CREATE OR REPLACE PACKAGE PKG_DEPARTAMENTO AS

    PROCEDURE altaDepartamento (
        p_nombre IN DEPARTAMENTO.NOMBRE%TYPE,
		p_idEmpresa IN DEPARTAMENTO.IDEMPRESA%TYPE,
		rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaDepartamento (
        p_idDepartamento IN DEPARTAMENTO.ID%TYPE,
        p_nombre IN DEPARTAMENTO.NOMBRE%TYPE,
		p_idEmpresa IN DEPARTAMENTO.IDEMPRESA%TYPE,
		rsp OUT LOGGER.RSP%TYPE
    );

END PKG_DEPARTAMENTO;
/

CREATE OR REPLACE PACKAGE BODY PKG_DEPARTAMENTO AS

    PROCEDURE altaDepartamento (
        p_nombre IN DEPARTAMENTO.NOMBRE%TYPE,
		p_idEmpresa IN DEPARTAMENTO.IDEMPRESA%TYPE,
		rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_DEPARTAMENTO.altaDepartamento';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
						
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_idEmpresa') := p_idEmpresa;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas            
			vParamAnalizado := 'p_idEmpresa';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
			
            
            -- Insertamos
            INSERT INTO DEPARTAMENTO (NOMBRE, IDEMPRESA)
                VALUES (p_nombre, p_idEmpresa)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Departamento creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaDepartamento;

    PROCEDURE actualizaDepartamento (
        p_idDepartamento IN DEPARTAMENTO.ID%TYPE,
        p_nombre IN DEPARTAMENTO.NOMBRE%TYPE,
		p_idEmpresa IN DEPARTAMENTO.IDEMPRESA%TYPE,
		rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_DEPARTAMENTO.actualizaDepartamento';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
		    
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idDepartamento';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idDepartamento, 'DEPARTAMENTO', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_idEmpresa') := p_idEmpresa;
			obligatoriosNumber('p_idDepartamento') := p_idDepartamento;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas            
			vParamAnalizado := 'p_idEmpresa';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
            
            -- Actualizamos
            UPDATE DEPARTAMENTO 
            SET NOMBRE = p_nombre,
                IDEMPRESA = p_idEmpresa                
            WHERE ID = p_idDepartamento;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Departamento actualizado con el id: ' || p_idDepartamento);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el departamento con el id: ' || p_idDepartamento);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaDepartamento;
    
END PKG_DEPARTAMENTO;

/
/**
************************* Auditoria departamento *************************
*/
CREATE OR REPLACE PACKAGE PKG_AUDITORIA_DEPARTAMENTO AS

    PROCEDURE asignaAuditoriaDepartamento (
        p_idAuditoria IN AUDITORIA_DEPARTAMENTO.IDAUDITORIA%TYPE,
		p_idDepartamento IN AUDITORIA_DEPARTAMENTO.IDDEPARTAMENTO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_AUDITORIA_DEPARTAMENTO;
/

CREATE OR REPLACE PACKAGE BODY PKG_AUDITORIA_DEPARTAMENTO AS

    PROCEDURE asignaAuditoriaDepartamento (
        p_idAuditoria IN AUDITORIA_DEPARTAMENTO.IDAUDITORIA%TYPE,
		p_idDepartamento IN AUDITORIA_DEPARTAMENTO.IDDEPARTAMENTO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_AUDITORIA_DEPARTAMENTO.asignaAuditoriaDepartamento';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_idAuditoria = ' || p_idAuditoria;
			parametrosEntrada := parametrosEntrada || ', p_idDepartamento = ' || p_idDepartamento;			
			
            -- Comprobacion de campos obligatorios
            obligatoriosNumber('p_idAuditoria') := p_idAuditoria;
			obligatoriosNumber('p_idDepartamento') := p_idDepartamento;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idAuditoria';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idAuditoria, 'AUDITORIA', FALSE);
			vParamAnalizado := 'p_idDepartamento';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idDepartamento, 'DEPARTAMENTO', FALSE);
			
            -- Insertamos
            INSERT INTO AUDITORIA_DEPARTAMENTO (IDAUDITORIA, IDDEPARTAMENTO)
                VALUES (p_idAuditoria, p_idDepartamento);
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'El departamento con el ID ' || p_idDepartamento || ' se asigno a la auditoria con el id ' || p_idAuditoria);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END asignaAuditoriaDepartamento;
    
END PKG_AUDITORIA_DEPARTAMENTO;
/

/**
************************* Control *************************
*/
CREATE OR REPLACE PACKAGE PKG_CONTROL AS

    PROCEDURE altaControl (
        p_descripcion IN CONTROL.DESCRIPCION%TYPE,
		p_idAuditoria IN CONTROL.IDAUDITORIA%TYPE,
        p_idCreador IN CONTROL.IDCREADOR%TYPE,
		p_idEjecutor IN CONTROL.IDEJECUTOR%TYPE,
		p_idRiesgo IN CONTROL.IDRIESGO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaControl (
        p_idControl IN CONTROL.ID%TYPE,
        p_descripcion IN CONTROL.DESCRIPCION%TYPE,
		p_idAuditoria IN CONTROL.IDAUDITORIA%TYPE,
        p_idCreador IN CONTROL.IDCREADOR%TYPE,
		p_idEjecutor IN CONTROL.IDEJECUTOR%TYPE,
		p_idRiesgo IN CONTROL.IDRIESGO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_CONTROL;
/

CREATE OR REPLACE PACKAGE BODY PKG_CONTROL AS

    PROCEDURE altaControl (
        p_descripcion IN CONTROL.DESCRIPCION%TYPE,
		p_idAuditoria IN CONTROL.IDAUDITORIA%TYPE,
        p_idCreador IN CONTROL.IDCREADOR%TYPE,
		p_idEjecutor IN CONTROL.IDEJECUTOR%TYPE,
		p_idRiesgo IN CONTROL.IDRIESGO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CONTROL.altaControl';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_idAuditoria = ' || p_idAuditoria;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idEjecutor = ' || p_idEjecutor;
			parametrosEntrada := parametrosEntrada || ', p_idRiesgo = ' || p_idRiesgo;
			
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosNumber('p_idAuditoria') := p_idAuditoria;
			obligatoriosNumber('p_idCreador') := p_idCreador;	
            obligatoriosNumber('p_idEjecutor') := p_idEjecutor;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idAuditoria';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idAuditoria, 'AUDITORIA', FALSE);
			vParamAnalizado := 'p_idCreador';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEjecutor';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEjecutor, 'PERSONA', FALSE);
			IF p_idRiesgo IS NOT NULL THEN
				vParamAnalizado := 'p_idRiesgo';
				PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idRiesgo, 'RIESGO', FALSE);
			END IF;
            
            -- Insertamos
            INSERT INTO CONTROL (DESCRIPCION, IDAUDITORIA, IDCREADOR, IDEJECUTOR, IDRIESGO)
                VALUES (p_descripcion, p_idAuditoria, p_idCreador, p_idEjecutor, p_idRiesgo)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Control creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaControl;

    PROCEDURE actualizaControl (
        p_idControl IN CONTROL.ID%TYPE,
        p_descripcion IN CONTROL.DESCRIPCION%TYPE,
		p_idAuditoria IN CONTROL.IDAUDITORIA%TYPE,
        p_idCreador IN CONTROL.IDCREADOR%TYPE,
		p_idEjecutor IN CONTROL.IDEJECUTOR%TYPE,
		p_idRiesgo IN CONTROL.IDRIESGO%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CONTROL.actualizaControl';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idControl = ' || p_idControl;
			parametrosEntrada := parametrosEntrada || ', p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_idAuditoria = ' || p_idAuditoria;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idEjecutor = ' || p_idEjecutor;
			parametrosEntrada := parametrosEntrada || ', p_idRiesgo = ' || p_idRiesgo;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idControl';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idControl, 'CONTROL', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
			obligatoriosNumber('p_idControl') := p_idControl;
            obligatoriosNumber('p_idAuditoria') := p_idAuditoria;
			obligatoriosNumber('p_idCreador') := p_idCreador;	
            obligatoriosNumber('p_idEjecutor') := p_idEjecutor;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idAuditoria';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idAuditoria, 'AUDITORIA', FALSE);
			vParamAnalizado := 'p_idCreador';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEjecutor';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEjecutor, 'PERSONA', FALSE);			
			IF p_idRiesgo IS NOT NULL THEN
				vParamAnalizado := 'p_idRiesgo';
				PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idRiesgo, 'RIESGO', FALSE);
			END IF;
            
            -- Actualizamos
            UPDATE CONTROL 
            SET DESCRIPCION = p_descripcion,
				IDAUDITORIA = p_idAuditoria,
                IDCREADOR = p_idCreador,
                IDEJECUTOR = p_idEjecutor,
                IDRIESGO = p_idRiesgo
            WHERE ID = p_idControl;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Control actualizado con el id: ' || p_idControl);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el control con el id: ' || p_idControl);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaControl;
    
END PKG_CONTROL;
/

/**
************************* Muestreo *************************
*/
CREATE OR REPLACE PACKAGE PKG_MUESTREO AS

    PROCEDURE altaMuestreo (
        p_descripcion IN MUESTREO.DESCRIPCION%TYPE,
		p_resultado IN MUESTREO.RESULTADO%TYPE,
        p_observaciones IN MUESTREO.OBSERVACIONES%TYPE,
		p_idControl IN MUESTREO.IDCONTROL%TYPE,
		p_idResponsable IN MUESTREO.IDRESPONSABLE%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaMuestreo (
        p_idMuestreo IN MUESTREO.ID%TYPE,
        p_descripcion IN MUESTREO.DESCRIPCION%TYPE,
		p_resultado IN MUESTREO.RESULTADO%TYPE,
        p_observaciones IN MUESTREO.OBSERVACIONES%TYPE,
		p_idControl IN MUESTREO.IDCONTROL%TYPE,
		p_idResponsable IN MUESTREO.IDRESPONSABLE%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_MUESTREO;
/

CREATE OR REPLACE PACKAGE BODY PKG_MUESTREO AS

    PROCEDURE altaMuestreo (
        p_descripcion IN MUESTREO.DESCRIPCION%TYPE,
		p_resultado IN MUESTREO.RESULTADO%TYPE,
        p_observaciones IN MUESTREO.OBSERVACIONES%TYPE,
		p_idControl IN MUESTREO.IDCONTROL%TYPE,
		p_idResponsable IN MUESTREO.IDRESPONSABLE%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_MUESTREO.altaMuestreo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
		tAux MUESTREO.RESULTADO%TYPE;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_resultado = ' || p_resultado;
			parametrosEntrada := parametrosEntrada || ', p_observaciones = ' || p_observaciones;
			parametrosEntrada := parametrosEntrada || ', p_idControl = ' || p_idControl;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            --obligatoriosString('p_resultado') := p_resultado;
            obligatoriosNumber('p_idControl') := p_idControl;
            obligatoriosNumber('p_idResponsable') := p_idResponsable;		
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idControl';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idControl, 'CONTROL', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
            
            tAux := p_resultado;
            IF p_resultado IS NULL THEN
                tAux := 'NO_EJECUTADO';
            END IF;
            
            -- Insertamos
            INSERT INTO MUESTREO (DESCRIPCION, RESULTADO, OBSERVACIONES, IDCONTROL, IDRESPONSABLE)
                VALUES (p_descripcion, tAux, p_observaciones, p_idControl, p_idResponsable)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Muestreo creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaMuestreo;

    PROCEDURE actualizaMuestreo (
        p_idMuestreo IN MUESTREO.ID%TYPE,
        p_descripcion IN MUESTREO.DESCRIPCION%TYPE,
		p_resultado IN MUESTREO.RESULTADO%TYPE,
        p_observaciones IN MUESTREO.OBSERVACIONES%TYPE,
		p_idControl IN MUESTREO.IDCONTROL%TYPE,
		p_idResponsable IN MUESTREO.IDRESPONSABLE%TYPE,		
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_MUESTREO.actualizaMuestreo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
		tAux MUESTREO.RESULTADO%TYPE;
        textoAux LOGGER.OBSERVACIONES%TYPE;
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idMuestreo = ' || p_idMuestreo;
			parametrosEntrada := parametrosEntrada || ', p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_resultado = ' || p_resultado;
			parametrosEntrada := parametrosEntrada || ', p_observaciones = ' || p_observaciones;
			parametrosEntrada := parametrosEntrada || ', p_idControl = ' || p_idControl;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			
			-- Comprobacion de clave primaria
			vParamAnalizado := 'p_idMuestreo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idMuestreo, 'MUESTREO', TRUE);
		   
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            --obligatoriosString('p_resultado') := p_resultado;
            obligatoriosNumber('p_idControl') := p_idControl;
            obligatoriosNumber('p_idResponsable') := p_idResponsable;		
			obligatoriosNumber('p_idMuestreo') := p_idMuestreo;		
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
			
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idControl';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idControl, 'CONTROL', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
            
            tAux := p_resultado;
            IF p_resultado IS NULL THEN
                tAux := 'NO_EJECUTADO';
            END IF;
            
            -- Actualizamos
            UPDATE MUESTREO 
            SET DESCRIPCION = p_descripcion,
				RESULTADO = tAux,
                OBSERVACIONES = p_observaciones,
                IDCONTROL = p_idControl,
                IDRESPONSABLE = p_idResponsable
            WHERE ID = p_idMuestreo;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Muestreo actualizado con el id: ' || p_idMuestreo);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el muestreo con el id: ' || p_idMuestreo);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaMuestreo;
    
END PKG_MUESTREO;
/

/**
************************* Persona *************************
*/
CREATE OR REPLACE PACKAGE PKG_PERSONA AS

    PROCEDURE altaPersona (
        p_nif IN PERSONA.NIF%TYPE,
		p_nombre IN PERSONA.NOMBRE%TYPE,
		p_idEmpresa IN PERSONA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN PERSONA.IDEMPRESAEXTERNA%TYPE,
		p_idDepartamento IN PERSONA.IDDEPARTAMENTO%TYPE,
		p_idOrigen IN PERSONA.IDORIGEN%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaPersona (
        p_idPersona IN PERSONA.ID%TYPE,
        p_nif IN PERSONA.NIF%TYPE,
		p_nombre IN PERSONA.NOMBRE%TYPE,
		p_idEmpresa IN PERSONA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN PERSONA.IDEMPRESAEXTERNA%TYPE,
		p_idDepartamento IN PERSONA.IDDEPARTAMENTO%TYPE,
		p_idOrigen IN PERSONA.IDORIGEN%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_PERSONA;
/

CREATE OR REPLACE PACKAGE BODY PKG_PERSONA AS

    PROCEDURE altaPersona (
        p_nif IN PERSONA.NIF%TYPE,
		p_nombre IN PERSONA.NOMBRE%TYPE,
		p_idEmpresa IN PERSONA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN PERSONA.IDEMPRESAEXTERNA%TYPE,
		p_idDepartamento IN PERSONA.IDDEPARTAMENTO%TYPE,
		p_idOrigen IN PERSONA.IDORIGEN%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_PERSONA.altaPersona';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nif = ' || p_nif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresaExterna = ' || p_idEmpresaExterna;
			parametrosEntrada := parametrosEntrada || ', p_idDepartamento = ' || p_idDepartamento;
			parametrosEntrada := parametrosEntrada || ', p_idOrigen = ' || p_idOrigen;
						
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nif') := p_nif;
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosNumber('p_idDepartamento') := p_idDepartamento;
			obligatoriosNumber('p_idOrigen') := p_idOrigen;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idOrigen';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idOrigen, 'EQUIPO_ORIGEN', FALSE);
			vParamAnalizado := 'p_idDepartamento';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idDepartamento, 'DEPARTAMENTO', FALSE);
            
            IF p_idEmpresa IS NOT NULL THEN
                vParamAnalizado := 'p_idEmpresa';
                PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
            END IF;
            
            IF p_idEmpresaExterna IS NOT NULL THEN
                vParamAnalizado := 'p_idEmpresaExterna';
                PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresaExterna, 'EMPRESA_EXTERNA', FALSE);
            END IF;
            
            -- Insertamos            
            INSERT INTO PERSONA (NIF, NOMBRE, IDEMPRESA, IDEMPRESAEXTERNA, IDDEPARTAMENTO, IDORIGEN)
                VALUES (p_nif, p_nombre, p_idEmpresa, p_idEmpresaExterna, p_idDepartamento, p_idOrigen)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Persona creada con el id: ' || vAux);
            
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaPersona;

    PROCEDURE actualizaPersona (
        p_idPersona IN PERSONA.ID%TYPE,
        p_nif IN PERSONA.NIF%TYPE,
		p_nombre IN PERSONA.NOMBRE%TYPE,
		p_idEmpresa IN PERSONA.IDEMPRESA%TYPE,
		p_idEmpresaExterna IN PERSONA.IDEMPRESAEXTERNA%TYPE,
		p_idDepartamento IN PERSONA.IDDEPARTAMENTO%TYPE,
		p_idOrigen IN PERSONA.IDORIGEN%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_PERSONA.actualizaPersona';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idPersona = ' || p_idPersona;
			parametrosEntrada := parametrosEntrada || ', p_nif = ' || p_nif;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresa = ' || p_idEmpresa;
			parametrosEntrada := parametrosEntrada || ', p_idEmpresaExterna = ' || p_idEmpresaExterna;
			parametrosEntrada := parametrosEntrada || ', p_idDepartamento = ' || p_idDepartamento;
			parametrosEntrada := parametrosEntrada || ', p_idOrigen = ' || p_idOrigen;
		    
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idPersona';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idPersona, 'PERSONA', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nif') := p_nif;
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosNumber('p_idDepartamento') := p_idDepartamento;
			obligatoriosNumber('p_idOrigen') := p_idOrigen;	
			obligatoriosNumber('p_idPersona') := p_idPersona;				
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
			
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idOrigen';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idOrigen, 'EQUIPO_ORIGEN', FALSE);
			vParamAnalizado := 'p_idDepartamento';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idDepartamento, 'DEPARTAMENTO', FALSE);
            
            IF p_idEmpresa IS NOT NULL THEN
                vParamAnalizado := 'p_idEmpresa';
                PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresa, 'EMPRESA', FALSE);
            END IF;
            
            IF p_idEmpresaExterna IS NOT NULL THEN
                vParamAnalizado := 'p_idEmpresaExterna';
                PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEmpresaExterna, 'EMPRESA_EXTERNA', FALSE);
            END IF;
            
            -- Actualizamos
            UPDATE PERSONA 
            SET NIF = p_nif,
				NOMBRE = p_nombre,
                IDORIGEN = p_idOrigen,
                IDEMPRESA = p_idEmpresa,
                IDEMPRESAEXTERNA = p_idEmpresaExterna,
                IDDEPARTAMENTO = p_idDepartamento
            WHERE ID = p_idPersona;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Persona actualizada con el id: ' || p_idPersona);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la persona con el id: ' || p_idPersona);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaPersona;
    
END PKG_PERSONA;
/

/**
************************* Acción *************************
*/
CREATE OR REPLACE PACKAGE PKG_ACCION AS

    PROCEDURE altaAccion (
        p_descripcion IN ACCION.DESCRIPCION%TYPE,
		p_fechaEstimadaEjecucion IN ACCION.FECHA_ESTIMADA_EJECUCION%TYPE,
		p_fechaRealEjecucion IN ACCION.FECHA_REAL_EJECUCION%TYPE,
        p_idCreador IN ACCION.IDCREADOR%TYPE,
        p_idResponsable IN ACCION.IDRESPONSABLE%TYPE,
		p_idEstado IN ACCION.IDESTADO%TYPE,
		p_idRiesgo IN ACCION.IDRIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaAccion (
        p_idAccion IN ACCION.ID%TYPE,
        p_descripcion IN ACCION.DESCRIPCION%TYPE,
		p_fechaEstimadaEjecucion IN ACCION.FECHA_ESTIMADA_EJECUCION%TYPE,
		p_fechaRealEjecucion IN ACCION.FECHA_REAL_EJECUCION%TYPE,
        p_idCreador IN ACCION.IDCREADOR%TYPE,
        p_idResponsable IN ACCION.IDRESPONSABLE%TYPE,
		p_idEstado IN ACCION.IDESTADO%TYPE,
		p_idRiesgo IN ACCION.IDRIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_ACCION;
/

CREATE OR REPLACE PACKAGE BODY PKG_ACCION AS

    PROCEDURE altaAccion (
        p_descripcion IN ACCION.DESCRIPCION%TYPE,
		p_fechaEstimadaEjecucion IN ACCION.FECHA_ESTIMADA_EJECUCION%TYPE,
		p_fechaRealEjecucion IN ACCION.FECHA_REAL_EJECUCION%TYPE,
        p_idCreador IN ACCION.IDCREADOR%TYPE,
        p_idResponsable IN ACCION.IDRESPONSABLE%TYPE,
		p_idEstado IN ACCION.IDESTADO%TYPE,
		p_idRiesgo IN ACCION.IDRIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ACCION.altaAccion';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_fechaEstimadaEjecucion = ' || p_fechaEstimadaEjecucion;
			parametrosEntrada := parametrosEntrada || ', p_fechaRealEjecucion = ' || p_fechaRealEjecucion;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			parametrosEntrada := parametrosEntrada || ', p_idEstado = ' || p_idEstado;
			parametrosEntrada := parametrosEntrada || ', p_idRiesgo = ' || p_idRiesgo;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosNumber('p_idCreador') := p_idCreador;
			obligatoriosNumber('p_idResponsable') := p_idResponsable;	
			obligatoriosNumber('p_idEstado') := p_idEstado;	
			obligatoriosNumber('p_idRiesgo') := p_idRiesgo;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCreador';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEstado';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstado, 'ESTADO_ACCION', FALSE);
			vParamAnalizado := 'p_idRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idRiesgo, 'RIESGO', FALSE);
            
            -- Insertamos
            INSERT INTO ACCION (DESCRIPCION, FECHA_CREACION, FECHA_ESTIMADA_EJECUCION, FECHA_REAL_EJECUCION, IDCREADOR, IDRESPONSABLE, IDESTADO, IDRIESGO)
                VALUES (p_descripcion, SYSDATE, p_fechaEstimadaEjecucion, p_fechaRealEjecucion, p_idCreador, p_idResponsable, p_idEstado, p_idRiesgo)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Acción creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaAccion;

    PROCEDURE actualizaAccion (
        p_idAccion IN ACCION.ID%TYPE,
        p_descripcion IN ACCION.DESCRIPCION%TYPE,
		p_fechaEstimadaEjecucion IN ACCION.FECHA_ESTIMADA_EJECUCION%TYPE,
		p_fechaRealEjecucion IN ACCION.FECHA_REAL_EJECUCION%TYPE,
        p_idCreador IN ACCION.IDCREADOR%TYPE,
        p_idResponsable IN ACCION.IDRESPONSABLE%TYPE,
		p_idEstado IN ACCION.IDESTADO%TYPE,
		p_idRiesgo IN ACCION.IDRIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ACCION.actualizaAccion';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idAccion = ' || p_idAccion;
			parametrosEntrada := parametrosEntrada || ', p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_fechaEstimadaEjecucion = ' || p_fechaEstimadaEjecucion;
			parametrosEntrada := parametrosEntrada || ', p_fechaRealEjecucion = ' || p_fechaRealEjecucion;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			parametrosEntrada := parametrosEntrada || ', p_idEstado = ' || p_idEstado;
			parametrosEntrada := parametrosEntrada || ', p_idRiesgo = ' || p_idRiesgo;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idAccion';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idAccion, 'ACCION', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosNumber('p_idCreador') := p_idCreador;
			obligatoriosNumber('p_idResponsable') := p_idResponsable;	
			obligatoriosNumber('p_idEstado') := p_idEstado;	
			obligatoriosNumber('p_idRiesgo') := p_idRiesgo;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCreador';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEstado';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstado, 'ESTADO_ACCION', FALSE);
			vParamAnalizado := 'p_idRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idRiesgo, 'RIESGO', FALSE);
            
            -- Actualizamos
            UPDATE ACCION 
            SET DESCRIPCION = p_descripcion,
				FECHA_ESTIMADA_EJECUCION = p_fechaEstimadaEjecucion,
				FECHA_REAL_EJECUCION = p_fechaRealEjecucion,
				IDCREADOR = p_idCreador,
				IDRESPONSABLE = p_idResponsable,
				IDESTADO = p_idEstado,
				IDRIESGO = p_idRiesgo
            WHERE ID = p_idAccion;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Acción actualizada con el id: ' || p_idAccion);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la acción con el id: ' || p_idAccion);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaAccion;
    
END PKG_ACCION;
/

/**
************************* Estado Acción *************************
*/
CREATE OR REPLACE PACKAGE PKG_ESTADO_ACCION AS

    PROCEDURE altaEstadoAccion (
        p_nombre IN ESTADO_ACCION.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaEstadoAccion (
        p_idEstadoAccion IN ESTADO_ACCION.ID%TYPE,
        p_nombre IN ESTADO_ACCION.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_ESTADO_ACCION;
/

CREATE OR REPLACE PACKAGE BODY PKG_ESTADO_ACCION AS

    PROCEDURE altaEstadoAccion (
        p_nombre IN ESTADO_ACCION.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ESTADO_ACCION.altaEstadoAccion';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO ESTADO_ACCION (NOMBRE)
                VALUES (p_nombre)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Estado acción creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaEstadoAccion;

    PROCEDURE actualizaEstadoAccion (
        p_idEstadoAccion IN ESTADO_ACCION.ID%TYPE,
        p_nombre IN ESTADO_ACCION.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ESTADO_ACCION.actualizaEstadoAccion';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idEstadoAccion = ' || p_idEstadoAccion;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idEstadoAccion';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstadoAccion, 'ESTADO_ACCION', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;	
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE ESTADO_ACCION 
            SET NOMBRE = p_nombre
            WHERE ID = p_idEstadoAccion;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Estado acción actualizado con el id: ' || p_idEstadoAccion);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el estado acción con el id: ' || p_idEstadoAccion);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaEstadoAccion;
    
END PKG_ESTADO_ACCION;
/

/**
************************* Equipo origen *************************
*/
CREATE OR REPLACE PACKAGE PKG_EQUIPO_ORIGEN AS

    PROCEDURE altaEquipoOrigen (
        p_nombre IN EQUIPO_ORIGEN.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaEquipoOrigen (
        p_idEquipoOrigen IN EQUIPO_ORIGEN.ID%TYPE,
        p_nombre IN EQUIPO_ORIGEN.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_EQUIPO_ORIGEN;
/

CREATE OR REPLACE PACKAGE BODY PKG_EQUIPO_ORIGEN AS

    PROCEDURE altaEquipoOrigen (
        p_nombre IN EQUIPO_ORIGEN.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EQUIPO_ORIGEN.altaEquipoOrigen';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO EQUIPO_ORIGEN (NOMBRE)
                VALUES (p_nombre)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Equipo origen creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaEquipoOrigen;

    PROCEDURE actualizaEquipoOrigen (
        p_idEquipoOrigen IN EQUIPO_ORIGEN.ID%TYPE,
        p_nombre IN EQUIPO_ORIGEN.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_EQUIPO_ORIGEN.actualizaEquipoOrigen';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idEquipoOrigen = ' || p_idEquipoOrigen;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idEquipoOrigen';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEquipoOrigen, 'EQUIPO_ORIGEN', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;	
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE EQUIPO_ORIGEN 
            SET NOMBRE = p_nombre
            WHERE ID = p_idEquipoOrigen;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Equipo origen actualizado con el id: ' || p_idEquipoOrigen);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el equipo origen con el id: ' || p_idEquipoOrigen);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaEquipoOrigen;
    
END PKG_EQUIPO_ORIGEN;
/

/**
************************* Categoría *************************
*/
CREATE OR REPLACE PACKAGE PKG_CATEGORIA AS

    PROCEDURE altaCategoria (
        p_nombre IN CATEGORIA.NOMBRE%TYPE,
		p_idCreador IN CATEGORIA.IDCREADOR%TYPE,
        p_idImportancia IN CATEGORIA.IDIMPORTANCIA%TYPE,	
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaCategoria (
        p_idCategoria IN CATEGORIA.ID%TYPE,
        p_nombre IN CATEGORIA.NOMBRE%TYPE,
		p_idCreador IN CATEGORIA.IDCREADOR%TYPE,
        p_idImportancia IN CATEGORIA.IDIMPORTANCIA%TYPE,	
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_CATEGORIA;
/

CREATE OR REPLACE PACKAGE BODY PKG_CATEGORIA AS

    PROCEDURE altaCategoria (
        p_nombre IN CATEGORIA.NOMBRE%TYPE,
		p_idCreador IN CATEGORIA.IDCREADOR%TYPE,
        p_idImportancia IN CATEGORIA.IDIMPORTANCIA%TYPE,	
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CATEGORIA.altaCategoria';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idImportancia = ' || p_idImportancia;
			
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosNumber('p_idCreador') := p_idCreador;
			obligatoriosNumber('p_idImportancia') := p_idImportancia;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCreador';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idImportancia';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idImportancia, 'IMPORTANCIA', FALSE);
            
            -- Insertamos
            INSERT INTO CATEGORIA (NOMBRE, FECHA_CREACION, IDCREADOR, IDIMPORTANCIA)
                VALUES (p_nombre, SYSDATE, p_idCreador, p_idImportancia)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Categoría creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaCategoria;

    PROCEDURE actualizaCategoria (
        p_idCategoria IN CATEGORIA.ID%TYPE,
        p_nombre IN CATEGORIA.NOMBRE%TYPE,
		p_idCreador IN CATEGORIA.IDCREADOR%TYPE,
        p_idImportancia IN CATEGORIA.IDIMPORTANCIA%TYPE,	
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CATEGORIA.actualizaCategoria';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idCategoria = ' || p_idCategoria;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_idCreador = ' || p_idCreador;
			parametrosEntrada := parametrosEntrada || ', p_idImportancia = ' || p_idImportancia;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idCategoria';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCategoria, 'CATEGORIA', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            obligatoriosNumber('p_idCreador') := p_idCreador;
			obligatoriosNumber('p_idImportancia') := p_idImportancia;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCreador';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCreador, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idImportancia';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idImportancia, 'IMPORTANCIA', FALSE);
            
            -- Actualizamos
            UPDATE CATEGORIA 
            SET NOMBRE = p_nombre,
				IDCREADOR = p_idCreador,
                IDIMPORTANCIA = p_idImportancia                
            WHERE ID = p_idCategoria;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Categoría actualizada con el id: ' || p_idCategoria);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la categoría con el id: ' || p_idCategoria);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaCategoria;
    
END PKG_CATEGORIA;
/

/**
************************* Importancia *************************
*/
CREATE OR REPLACE PACKAGE PKG_IMPORTANCIA AS

    PROCEDURE altaImportancia (
        p_nombre IN IMPORTANCIA.NOMBRE%TYPE,
        p_valor IN IMPORTANCIA.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaImportancia (
        p_idImportancia IN IMPORTANCIA.ID%TYPE,
        p_nombre IN IMPORTANCIA.NOMBRE%TYPE,
        p_valor IN IMPORTANCIA.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_IMPORTANCIA;
/

CREATE OR REPLACE PACKAGE BODY PKG_IMPORTANCIA AS

    PROCEDURE altaImportancia (
        p_nombre IN IMPORTANCIA.NOMBRE%TYPE,
        p_valor IN IMPORTANCIA.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_IMPORTANCIA.altaImportancia';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_valor = ' || p_valor;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_valor') := p_valor;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO IMPORTANCIA (NOMBRE, VALOR)
                VALUES (p_nombre, p_valor)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Importancia creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaImportancia;

    PROCEDURE actualizaImportancia (
        p_idImportancia IN IMPORTANCIA.ID%TYPE,
        p_nombre IN IMPORTANCIA.NOMBRE%TYPE,
        p_valor IN IMPORTANCIA.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_IMPORTANCIA.actualizaImportancia';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idImportancia = ' || p_idImportancia;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_valor = ' || p_valor;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idImportancia';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idImportancia, 'IMPORTANCIA', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;	
			obligatoriosNumber('p_valor') := p_valor;	
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE IMPORTANCIA 
            SET NOMBRE = p_nombre,
				VALOR = p_valor
            WHERE ID = p_idImportancia;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Importancia actualizada con el id: ' || p_idImportancia);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la importancia con el id: ' || p_idImportancia);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaImportancia;
    
END PKG_IMPORTANCIA;
/

/**
************************* Criticidad *************************
*/
CREATE OR REPLACE PACKAGE PKG_CRITICIDAD AS

    PROCEDURE altaCriticidad (
        p_nombre IN CRITICIDAD.NOMBRE%TYPE,
        p_valor IN CRITICIDAD.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaCriticidad (
        p_idCriticidad IN CRITICIDAD.ID%TYPE,
        p_nombre IN CRITICIDAD.NOMBRE%TYPE,
        p_valor IN CRITICIDAD.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_CRITICIDAD;
/

CREATE OR REPLACE PACKAGE BODY PKG_CRITICIDAD AS

    PROCEDURE altaCriticidad (
        p_nombre IN CRITICIDAD.NOMBRE%TYPE,
        p_valor IN CRITICIDAD.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CRITICIDAD.altaCriticidad';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_valor = ' || p_valor;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
			obligatoriosNumber('p_valor') := p_valor;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO CRITICIDAD (NOMBRE, VALOR)
                VALUES (p_nombre, p_valor)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Criticidad creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaCriticidad;

    PROCEDURE actualizaCriticidad (
        p_idCriticidad IN CRITICIDAD.ID%TYPE,
        p_nombre IN CRITICIDAD.NOMBRE%TYPE,
        p_valor IN CRITICIDAD.VALOR%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_CRITICIDAD.actualizaCriticidad';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idCriticidad = ' || p_idCriticidad;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
			parametrosEntrada := parametrosEntrada || ', p_valor = ' || p_valor;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idCriticidad';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCriticidad, 'CRITICIDAD', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;	
			obligatoriosNumber('p_valor') := p_valor;	
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE CRITICIDAD 
            SET NOMBRE = p_nombre,
				VALOR = p_valor
            WHERE ID = p_idCriticidad;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Criticidad actualizada con el id: ' || p_idCriticidad);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe la criticidad con el id: ' || p_idCriticidad);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaCriticidad;
    
END PKG_CRITICIDAD;
/

/**
************************* Estado riesgo *************************
*/
CREATE OR REPLACE PACKAGE PKG_ESTADO_RIESGO AS

    PROCEDURE altaEstadoRiesgo (
        p_nombre IN ESTADO_RIESGO.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaEstadoRiesgo (
        p_idEstadoRiesgo IN ESTADO_RIESGO.ID%TYPE,
        p_nombre IN ESTADO_RIESGO.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_ESTADO_RIESGO;
/

CREATE OR REPLACE PACKAGE BODY PKG_ESTADO_RIESGO AS

    PROCEDURE altaEstadoRiesgo (
        p_nombre IN ESTADO_RIESGO.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ESTADO_RIESGO.altaEstadoRiesgo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_nombre = ' || p_nombre;
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Insertamos
            INSERT INTO ESTADO_RIESGO (NOMBRE)
                VALUES (p_nombre)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Estado riesgo creada con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaEstadoRiesgo;

    PROCEDURE actualizaEstadoRiesgo (
        p_idEstadoRiesgo IN ESTADO_RIESGO.ID%TYPE,
        p_nombre IN ESTADO_RIESGO.NOMBRE%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_ESTADO_RIESGO.actualizaEstadoRiesgo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idEstadoRiesgo = ' || p_idEstadoRiesgo;
			parametrosEntrada := parametrosEntrada || ', p_nombre = ' || p_nombre;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idEstadoRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstadoRiesgo, 'ESTADO_RIESGO', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_nombre') := p_nombre;
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Actualizamos
            UPDATE ESTADO_RIESGO 
            SET NOMBRE = p_nombre
            WHERE ID = p_idEstadoRiesgo;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Estado riesgo actualizado con el id: ' || p_idEstadoRiesgo);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el estado riesgo con el id: ' || p_idEstadoRiesgo);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaEstadoRiesgo;
    
END PKG_ESTADO_RIESGO;
/

/**
************************* Riesgo *************************
*/
CREATE OR REPLACE PACKAGE PKG_RIESGO AS

    PROCEDURE altaRiesgo (
        p_descripcion IN RIESGO.DESCRIPCION%TYPE,
		p_idCategoria IN RIESGO.IDCATEGORIA%TYPE,
        p_idCriticidad IN RIESGO.IDCRITICIDAD%TYPE,
		p_idOrigen IN RIESGO.IDORIGEN%TYPE,
		p_idResponsable IN RIESGO.IDRESPONSABLE%TYPE,
		p_idEstadoRiesgo IN RIESGO.IDESTADORIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

    PROCEDURE actualizaRiesgo (
        p_idRiesgo IN RIESGO.ID%TYPE,
        p_descripcion IN RIESGO.DESCRIPCION%TYPE,
		p_idCategoria IN RIESGO.IDCATEGORIA%TYPE,
        p_idCriticidad IN RIESGO.IDCRITICIDAD%TYPE,
		p_idOrigen IN RIESGO.IDORIGEN%TYPE,
		p_idResponsable IN RIESGO.IDRESPONSABLE%TYPE,
		p_idEstadoRiesgo IN RIESGO.IDESTADORIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    );

END PKG_RIESGO;
/

CREATE OR REPLACE PACKAGE BODY PKG_RIESGO AS

    PROCEDURE altaRiesgo (
        p_descripcion IN RIESGO.DESCRIPCION%TYPE,
		p_idCategoria IN RIESGO.IDCATEGORIA%TYPE,
        p_idCriticidad IN RIESGO.IDCRITICIDAD%TYPE,
		p_idOrigen IN RIESGO.IDORIGEN%TYPE,
		p_idResponsable IN RIESGO.IDRESPONSABLE%TYPE,
		p_idEstadoRiesgo IN RIESGO.IDESTADORIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_RIESGO.altaRiesgo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
		
			parametrosEntrada := 'p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_idCategoria = ' || p_idCategoria;
			parametrosEntrada := parametrosEntrada || ', p_idCriticidad = ' || p_idCriticidad;
			parametrosEntrada := parametrosEntrada || ', p_idOrigen = ' || p_idOrigen;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			parametrosEntrada := parametrosEntrada || ', p_idEstadoRiesgo = ' || p_idEstadoRiesgo;
			
			
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosNumber('p_idCategoria') := p_idCategoria;
			obligatoriosNumber('p_idCriticidad') := p_idCriticidad;			
			obligatoriosNumber('p_idOrigen') := p_idOrigen;			
			obligatoriosNumber('p_idResponsable') := p_idResponsable;	
			obligatoriosNumber('p_idEstadoRiesgo') := p_idEstadoRiesgo;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCategoria';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCategoria, 'CATEGORIA', FALSE);
			vParamAnalizado := 'p_idCriticidad';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCriticidad, 'CRITICIDAD', FALSE);
			vParamAnalizado := 'p_idOrigen';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idOrigen, 'EQUIPO_ORIGEN', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEstadoRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstadoRiesgo, 'ESTADO_RIESGO', FALSE);
            
            -- Insertamos
            INSERT INTO RIESGO (DESCRIPCION, FECHA_CREACION, IDCATEGORIA, IDCRITICIDAD, IDORIGEN, IDRESPONSABLE, IDESTADORIESGO)
                VALUES (p_descripcion, SYSDATE, p_idCategoria, p_idCriticidad, p_idOrigen, p_idResponsable, p_idEstadoRiesgo)
            RETURNING ID into vAux;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Riesgo creado con el id: ' || vAux);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END altaRiesgo;

    PROCEDURE actualizaRiesgo (
        p_idRiesgo IN RIESGO.ID%TYPE,
        p_descripcion IN RIESGO.DESCRIPCION%TYPE,
		p_idCategoria IN RIESGO.IDCATEGORIA%TYPE,
        p_idCriticidad IN RIESGO.IDCRITICIDAD%TYPE,
		p_idOrigen IN RIESGO.IDORIGEN%TYPE,
		p_idResponsable IN RIESGO.IDRESPONSABLE%TYPE,
		p_idEstadoRiesgo IN RIESGO.IDESTADORIESGO%TYPE,
        rsp OUT LOGGER.RSP%TYPE
    )
    AS
        nombreProcedimiento LOGGER.NOMBRE_PROCEDIMIENTO%TYPE := 'PKG_RIESGO.actualizaRiesgo';
        parametrosEntrada LOGGER.ENTRADA%TYPE;
        vParamAnalizado VARCHAR2(50);
        vAux NUMBER;
        textoAux LOGGER.OBSERVACIONES%TYPE;
		
        obligatoriosString PKG_UTILIDADES.ARRAY_STRING;
        obligatoriosNumber PKG_UTILIDADES.ARRAY_NUMBER;
        obligatoriosDate PKG_UTILIDADES.ARRAY_DATE;
        
        BEGIN
			parametrosEntrada := 'p_idRiesgo = ' || p_idRiesgo;
			parametrosEntrada := parametrosEntrada || ', p_descripcion = ' || p_descripcion;
			parametrosEntrada := parametrosEntrada || ', p_idCategoria = ' || p_idCategoria;
			parametrosEntrada := parametrosEntrada || ', p_idCriticidad = ' || p_idCriticidad;
			parametrosEntrada := parametrosEntrada || ', p_idOrigen = ' || p_idOrigen;
			parametrosEntrada := parametrosEntrada || ', p_idResponsable = ' || p_idResponsable;
			parametrosEntrada := parametrosEntrada || ', p_idEstadoRiesgo = ' || p_idEstadoRiesgo;
		
            -- Comprobacion de clave primaria
			vParamAnalizado := 'p_idRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idRiesgo, 'RIESGO', TRUE);
            
            -- Comprobacion de campos obligatorios
            obligatoriosString('p_descripcion') := p_descripcion;
            obligatoriosNumber('p_idCategoria') := p_idCategoria;
			obligatoriosNumber('p_idCriticidad') := p_idCriticidad;			
			obligatoriosNumber('p_idOrigen') := p_idOrigen;			
			obligatoriosNumber('p_idResponsable') := p_idResponsable;	
			obligatoriosNumber('p_idEstadoRiesgo') := p_idEstadoRiesgo;			
            PKG_UTILIDADES.compruebaCamposObligatorios(obligatoriosString, obligatoriosNumber, obligatoriosDate, vParamAnalizado);
            IF vParamAnalizado IS NOT NULL THEN
                RAISE PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS;
            END IF;
            
            -- Comprobacion de claves foraneas
            vParamAnalizado := 'p_idCategoria';
            PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCategoria, 'CATEGORIA', FALSE);
			vParamAnalizado := 'p_idCriticidad';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idCriticidad, 'CRITICIDAD', FALSE);
			vParamAnalizado := 'p_idOrigen';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idOrigen, 'EQUIPO_ORIGEN', FALSE);
			vParamAnalizado := 'p_idResponsable';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idResponsable, 'PERSONA', FALSE);
			vParamAnalizado := 'p_idEstadoRiesgo';
			PKG_UTILIDADES.compruebaExistenciaIDEnTabla(p_idEstadoRiesgo, 'ESTADO_RIESGO', FALSE);
            
            -- Actualizamos
            UPDATE RIESGO 
            SET DESCRIPCION = p_descripcion,
                IDCATEGORIA = p_idCategoria,
                IDCRITICIDAD = p_idCriticidad,
                IDORIGEN = p_idOrigen,
                IDRESPONSABLE = p_idResponsable,
				IDESTADORIESGO = p_idEstadoRiesgo
            WHERE ID = p_idRiesgo;
            
            rsp := 'OK';
            PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Riesgo actualizado con el id: ' || p_idRiesgo);
        
        EXCEPTION
            WHEN PKG_UTILIDADES.EX_PARAMETROS_OBLIGATORIOS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PARAMETROS_OBLIGATORIOS;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Falta campo: ' || vParamAnalizado);
            
            WHEN PKG_UTILIDADES.EX_PK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_PK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'No existe el riesgo con el id: ' || p_idRiesgo);
                
            WHEN PKG_UTILIDADES.EX_FK_NO_ENCONTRADA THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_FK_NO_ENCONTRADA;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, 'Campo de clave foránea: ' || vParamAnalizado);
                
            WHEN OTHERS THEN
                rsp := PKG_UTILIDADES.TEXTO_EX_EXCEPCION_NO_CONTROLADA;
                textoAux := 'Código error: ' || SQLCODE || ' Mensaje de error: ' || SQLERRM;
                PKG_UTILIDADES.addLog(nombreProcedimiento, parametrosEntrada, rsp, textoAux);
            
    END actualizaRiesgo;
    
END PKG_RIESGO;
/

