/**
* Consultas de comprobaciones de repositorio estadístico.
* Por cada requisito se realizan dos consultas, para comprobar que el agrupado de los datos da el mismo resultado que los datos existentes en las tablas del repositorio estadístico.
* La primera es sobre las tablas de la aplicación, realizando agrupaciones y sumatorias, estas consultas no son de tiempo constante igual a 1.
* La segunda es sobre las tablas del repositorio estadístico, estas consultas si que son de tiempo constante igual a 1.
*/
ALTER SESSION SET CURRENT_SCHEMA = TFG_GPOSESMZ;  
SET SERVEROUTPUT ON;

/*
* En el momento de ejecutar la consulta, porcentaje de riesgos no corregidos o mitigados con un impacto menor a 4
*/
DECLARE
    TOTAL_MENOR_4 NUMBER;
    ABIERTOS_MENOR_4 NUMBER;
    PORCENTAJE NUMBER;
BEGIN
    SELECT COUNT(*) 
        INTO  TOTAL_MENOR_4 
    FROM RIESGO 
    WHERE IMPACTO < 4;
    
    SELECT COUNT(*) 
        INTO ABIERTOS_MENOR_4
    FROM RIESGO 
    WHERE IMPACTO < 4 AND IDESTADORIESGO = (SELECT ID FROM ESTADO_RIESGO WHERE NOMBRE = 'Abierto');
    
    PORCENTAJE := ROUND((ABIERTOS_MENOR_4 * 100) / TOTAL_MENOR_4, 2);
    DBMS_OUTPUT.PUT_LINE('Número de riesgos con impacto menor de 4: ' || TOTAL_MENOR_4);
    DBMS_OUTPUT.PUT_LINE('Número de riesgos ABIERTOS con impacto menor de 4: ' || ABIERTOS_MENOR_4);
    DBMS_OUTPUT.PUT_LINE('Porcentaje de riesgos ABIERTOS con impacto menor de 4: ' || PORCENTAJE);

END;
/

SELECT PORCENTAJERIESGOSABIERTOSIMPACTOMENOR4 FROM REP_GENERAL; 

/*
* Dado un año en concreto, número total de riesgos con un impacto de 1
*/
SELECT extract( year from fecha_creacion) anho, COUNT(*) 
FROM RIESGO
WHERE IMPACTO = 1
GROUP BY extract( year from fecha_creacion)
ORDER BY 1 DESC;

select ANHO, NUMRIESGOSIMPACTO1
FROM REP_EST_ANHO_RIESGO
ORDER BY 1 DESC;

/*
* En el último año finalizado, número de riesgos que actualmente aún estan en estado de abierto
*/

SELECT extract( year from fecha_creacion) ANHO, COUNT(*) 
FROM RIESGO 
WHERE IDESTADORIESGO = (SELECT ID FROM ESTADO_RIESGO WHERE NOMBRE = 'Abierto')
GROUP BY extract( year from fecha_creacion)
ORDER BY 1 DESC;

SELECT ANHO, NUMRIESGOSESTADOABIERTO
FROM REP_EST_ANHO_RIESGO
ORDER BY 1 DESC;

/*
* Con todos los datos disponibles en la BD, departamento con un número mayor de riesgos detectados por auditorías externas
*/
SELECT NOMBRE FROM (
    SELECT D.NOMBRE, COUNT(*) TOTAL FROM RIESGO R
        INNER JOIN PERSONA P ON P.ID = R.IDRESPONSABLE
        INNER JOIN DEPARTAMENTO D ON D.ID = P.IDDEPARTAMENTO
    WHERE R.IDORIGEN = (SELECT ID FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Auditoria externa')
    GROUP BY D.NOMBRE
    ORDER BY total desc)
WHERE ROWNUM = 1;

SELECT DEPTMASRIESGOSEQUIPOEXTERNO FROM REP_GENERAL;
/*
* En el año actual, número total de acciones en estado de en curso
*/
SELECT extract( year from fecha_creacion) anho, count(*) FROM ACCION
WHERE IDESTADO = (SELECT ID FROM ESTADO_ACCION WHERE NOMBRE = 'En curso')
AND extract( year from fecha_creacion) = EXTRACT (YEAR FROM SYSDATE)
group by extract( year from fecha_creacion)
order by 1 desc;

SELECT anho, NUMACCIONESESTADOCURSO FROM REP_EST_ANHO_RIESGO
WHERE anho = EXTRACT (YEAR FROM SYSDATE);

/*
* Dado un año en concreto, diferencia entre el número de riesgos de impacto 1 detectados por auditoría interna y externa
*/
DECLARE
    INTERNOS_IMPACTO_1 NUMBER;
    EXTERNOS_IMPACTO_1 NUMBER;
    CURSOR c_ANHOS IS
        SELECT distinct extract( year from fecha_creacion) anho FROM RIESGO ORDER BY 1 DESC;
BEGIN
    FOR fila in c_ANHOS
    LOOP
        SELECT COUNT(*) 
            INTO INTERNOS_IMPACTO_1
        FROM RIESGO
        WHERE IMPACTO = 1 AND IDORIGEN = (SELECT ID FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Auditoria interna')
        AND extract( year from fecha_creacion) = fila.anho;
        
        SELECT COUNT(*) 
            INTO EXTERNOS_IMPACTO_1
        FROM RIESGO
        WHERE IMPACTO = 1 AND IDORIGEN = (SELECT ID FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Auditoria externa')
        AND extract( year from fecha_creacion) = fila.anho;
        
        DBMS_OUTPUT.PUT_LINE('La diferencia entre el número de riesgos de impacto 1 detectados por auditoria interna y externa para el año ' || fila.anho || ' es de: ' || (INTERNOS_IMPACTO_1 - EXTERNOS_IMPACTO_1) );
    
    END LOOP;
END;
/

SELECT ANHO, DIFRIESGOSIMPACTO1INTEREXTER 
FROM REP_EST_ANHO_RIESGO
ORDER BY 1 DESC;

/*
* En el momento de ejecutar la consulta, persona de la empresa que tiene más acciones abiertas (en estado de en curso o de definida) asignadas
*/
SELECT NOMBRE 
FROM (
    SELECT P.NOMBRE, COUNT(*) FROM ACCION A
        INNER JOIN PERSONA P ON P.ID = A.IDRESPONSABLE
    WHERE A.IDESTADO IN (SELECT ID FROM ESTADO_ACCION WHERE NOMBRE IN ('Definida','En curso'))
    GROUP BY P.NOMBRE
    ORDER BY 2 DESC
)
WHERE ROWNUM = 1;

SELECT PERSONAMASACCIONESABIERTASASIGNADAS FROM REP_GENERAL;

/*
* Teniendo en cuenta el año en curso y el anterior, número de riesgos de ciberseguridad detectados por autoevaluación de los departamentos
*/
SELECT COUNT(*) FROM RIESGO
WHERE IDCATEGORIA = (SELECT ID FROM CATEGORIA WHERE NOMBRE = 'Ciberseguridad')
    AND IDORIGEN = (SELECT ID FROM EQUIPO_ORIGEN WHERE NOMBRE = 'Autoevaluación')
    AND FECHA_CREACION >= TO_DATE('01/01/' || (to_char(sysdate, 'YYYY') -1) ,'DD/MM/YYYY');

SELECT NUMRIESGCIBERAUTOEVAACTANT FROM REP_GENERAL;
/*
* En el último año, coste medio de todas las auditorías externas realizadas
*/
DECLARE
    NUM_TOTAL_AUDITORIAS_EXTERNAS NUMBER;
    SUMA_TOTAL_AUDITORIAS_CON_COSTE NUMBER;
    COSTE_MEDIO NUMBER(10,2);
	
	CURSOR c_anhos IS
		SELECT DISTINCT EXTRACT (YEAR FROM FECHA_INICIO) ANHO FROM AUDITORIA;
BEGIN
	
	FOR fila IN c_anhos
	LOOP	
		SELECT COUNT(*)
			INTO NUM_TOTAL_AUDITORIAS_EXTERNAS
		FROM AUDITORIA
		WHERE EXTRACT (YEAR FROM FECHA_INICIO) = fila.ANHO
		AND COSTE <> 0;
		
		SELECT SUM(COSTE)
			INTO SUMA_TOTAL_AUDITORIAS_CON_COSTE
		FROM AUDITORIA
		WHERE EXTRACT (YEAR FROM FECHA_INICIO) = fila.ANHO
		AND COSTE <> 0;
			
		BEGIN
			COSTE_MEDIO := SUMA_TOTAL_AUDITORIAS_CON_COSTE / NUM_TOTAL_AUDITORIAS_EXTERNAS;
		EXCEPTION WHEN OTHERS THEN
			COSTE_MEDIO := NULL;
		END;
		
		DBMS_OUTPUT.PUT_LINE('El coste medio de las auditorias externas realizadas en el año ' || fila.ANHO || ' es de: ' || COSTE_MEDIO);
	
	END LOOP;
END;
/

SELECT ANHO, COSTEMEDIOAUDITORIASEXTERNAS
FROM REP_EST_ANHO_RIESGO;

/*
* Teniendo en cuenta todos los datos disponibles, año con un número mayor de acciones que al final del año en cuestión estaban en estado de implementada con el riesgo mitigado
*/
SELECT ANHO
        FROM (
            SELECT extract( year from fecha_creacion) ANHO, count(*) numero
            FROM ACCION
            WHERE IDESTADO = 4
            GROUP BY extract( year from fecha_creacion)
            ORDER BY numero desc
        )
        WHERE ROWNUM = 1;
        
SELECT ANHOMASACCIONIMPLEMRIESGMITIG FROM REP_GENERAL;

/*
* Teniendo en cuenta sólo el último año finalizado, Top3 de acciones definidas teniendo en cuenta el tiempo que estuvieron abiertas (desde su creación hasta que pasen a uno de loss estados de implementada). Se debe indicar el nombre de las 3 acciones que estuvieron más tiempo abiertas
*/
DECLARE
    top3 VARCHAR2(2000);
    ultimoAnhoFinalizado NUMBER;
BEGIN
    ultimoAnhoFinalizado := EXTRACT (YEAR FROM SYSDATE) - 1;
    
    SELECT LISTAGG(DESCRIPCION, ', ') 
        INTO TOP3
        FROM(
            select AC.DESCRIPCION, aeh.*, ROUND(aeh.fechaestado - aeh.fechaestadoanterior)
            FROM ACCION_ESTADO_HISTORICO AEH
                INNER JOIN ACCION AC ON AC.ID = AEH.IDACCION
            WHERE AC.FECHA_CREACION >= TO_DATE('01/01/'||ultimoAnhoFinalizado, 'DD/MM/YYYY') AND AC.FECHA_CREACION <= TO_DATE('31/12/'||ultimoAnhoFinalizado, 'DD/MM/YYYY')
                AND IDESTADOACTUAL IN (3,4)
                AND IDESTADOANTERIOR = 1
            ORDER BY (aeh.fechaestado - aeh.fechaestadoanterior) DESC
            )
        WHERE ROWNUM <=3;
        DBMS_OUTPUT.PUT_LINE('TOP3 de acciones definidas teniendo en cuenta el tiempo que estuvieron abiertas. Para el año '|| ultimoAnhoFinalizado || ': '||TOP3);
END;
/

SELECT TOP3NOMBREACCIONESMAXTIEMPOABIERT FROM REP_GENERAL;

/*
* En el momento de ejecutar la consulta, número de riesgos de categoría 1 (en cualquier estado)
*/
SELECT COUNT(*)
FROM RIESGO
WHERE IDCATEGORIA = 1;

SELECT NUMERORIESGOSCATEGORIA1 FROM REP_GENERAL;

/*
* Dado un año en concreto, porcentaje de acciones descartadas.
*/
DECLARE
	NUM_TOTAL_ACCIONES NUMBER;
	NUM_TOTAL_ACCIONES_DESCARTADAS NUMBER;
	CURSOR c_ANHOS IS
        SELECT distinct extract( year from fecha_creacion) anho FROM ACCION ORDER BY 1 DESC;
BEGIN

	FOR fila IN c_ANHOS
	LOOP
		SELECT COUNT(*) 
			INTO NUM_TOTAL_ACCIONES
		FROM ACCION
            WHERE FECHA_CREACION >= TO_DATE('01/01'||fila.anho,'DD/MM/YYYY') AND FECHA_CREACION < TO_DATE('01/01'||(fila.anho+1),'DD/MM/YYYY');
		
		SELECT COUNT(*)
			INTO NUM_TOTAL_ACCIONES_DESCARTADAS
		FROM ACCION
		WHERE IDESTADO = 5
        AND FECHA_CREACION >= TO_DATE('01/01'||fila.anho,'DD/MM/YYYY') AND FECHA_CREACION < TO_DATE('01/01'||(fila.anho+1),'DD/MM/YYYY');
		
		DBMS_OUTPUT.PUT_LINE('El porcentaje de acciones descartadas para el año ' || fila.anho || ' es ' || ROUND(((NUM_TOTAL_ACCIONES_DESCARTADAS * 100) / NUM_TOTAL_ACCIONES), 2) || ' %');
	END LOOP;
END;
/

SELECT PORCENTAJEACCIONESDESCARTADAS FROM REP_EST_ANHO_RIESGO;